package checkpoint2html;

use strict;
use CheckPoint;
use Constants;
use Options;
use Sections2html;
use object2html;
use Strings;

require Exporter;
our (@ISA, @EXPORT);
@ISA = qw(Exporter);
@EXPORT = qw(checkpoint2html);

# Initialize special marker signs ...
my $Check      = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/check.png>';
my $CheckTick  = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/checktick.png>';
my $Mark       = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/mark.png>';
my $MarkDot    = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/markdot.png>';
my $Spacer     = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/spacer.png>';
my $None       = '<I>(none)</I>';

##########################################################################
# Routine: checkpoint2html
#
# Description:
# Routine to create the CHECKPOINT table
##########################################################################
sub checkpoint2html {
	use strict;

	my $Policy   = shift;
	my $template = shift;
	my $CPtable  = shift;
	my (%layout, $version);
	my $obj      = shift;
	my @ObjList  = @{$obj};

	# Global variables within scope of checkpoint2html ...
	our @tobj = ();	# Holds the temp table of HTML code

	my $cnt;	# Holds the count of objects processed
	my ($obj, $name, $type, $current_type);
	
	# Determine layout of CheckPoint table ...
	%layout = (
		'IP Address'				=> 'GENERAL',
		'General Properties'		=> 'PRODUCTS',
		'System Interfaces'			=> 'PRODUCTS',
		'Cluster Members'			=> 'PRODUCTS',
		'3rd Party Configuration'	=> 'PRODUCTS',
		'ClusterXL'					=> 'PRODUCTS',
		'Sofaware Settings'			=> 'PRODUCTS',
		'Synchronization'			=> 'PROP1',
		'Topology'					=> 'PROP1',
		'NAT'						=> 'PROP1',
		'VPN'						=> 'PROP2',
		'Remote Access'				=> 'PROP1',
		'Authentication'			=> 'PROP1',
		'SmartDirectory'			=> 'PRODUCTS',
		'SmartView Monitor'			=> 'PRODUCTS',
		'UserAuthority Server'		=> 'PRODUCTS',
		'UserAuthority WebAccess'	=> 'PRODUCTS',
		'WebServer'					=> 'PRODUCTS',
		'MailServer'				=> 'PRODUCTS',
		'DNSServer'					=> 'PRODUCTS',
		'FireWall-1 GX'				=> 'PRODUCTS',
		'Logs and Masters'			=> 'PROP2',
		'Capacity Optimization'		=> 'PROP2',
		'Content Filtering'         => 'PRODUCTS',
		'Advanced'					=> 'PROP2'
	);

	# Determine version of database ...
	$version = GetKey('FW1Objects.version');
	if ($version le '5.0') { 
		$version = 'NG';
	} elsif ($version gt '5.0') {
		$version = 'NGX';
	}

	#----------------------------------------------------------------
	#---                  CHECKPOINT to HTML                      ---
	#----------------------------------------------------------------
	$cnt = 0;
	$current_type = '';
	foreach $name (sort {uc($a) cmp uc($b)} @ObjList) {
		(undef, $type, $name) = split(/:/, $name);
		$obj  = GetKey($CPtable, $name);

		# Determine if type has changed ...
		if ($type ne $current_type) {
			$tobj[$cnt]{TYPE} = 1;
			$tobj[$cnt]{OBJ_TYPE} = $type;
			$current_type = $type;
		}

		# Output object to HTML; the return value is the number
		# of object lines created (members of the object)...
		$cnt += cpobject2html($obj, $type, $name, $cnt, 0);
	}

	# Parse the CheckPoint objects trough the templating engine ...
	$template->param(CHECKPOINT=>\@tobj);

	#----------------------------------------------------------------
	#---              END of CHECKPOINT to HTML                   ---
	#----------------------------------------------------------------

	

    ##########################################################################
    # Routine: cpobject2html
    #
    # Description:
    # Create the HTML entry of one single checkpoint object
    #  in the CHECKPOINT table.
    #
	# Global variables used in global context (within scope of checkpoint2html):
	# %layout	Layout of the checkpoint table ...
	# @tobj		Array of objects to fill ...
	# $CPtable	CheckPoint table to be used ...
	#
	# Input variables:
	# $obj		Reference to the object to process
	# $type		Type of the object (determines which sections to process
	# $name		Name of the object
	# $cnt		Number of the last created element in the array $tobj
	# $level	Number of indents to use in case of members of an object,
	#			or the members of the members of the object
	#
	# Return value:
	# $cpobjecthtml	The number of elements written to the array $tobj
    ##########################################################################
    sub cpobject2html {
    	use strict;
    	my $obj  = shift;	# Ref to object in database
    	my $type = shift;	# Type of object to be displayed
    	my $name = shift;	# Name of object
		my $cnt  = shift;	# row of object in table to be written
		my $level= shift;	# depth of recursion

		# Local variables ...
    	my ($section, $layout);
    	my ($var, %var, @var, $val);
    	
		my $cpversion = Version($obj);							# Version number of the object $obj
		my $class     = GetKey($obj, "AdminInfo:ClassName");	# Classname of the object
		my $members = 0;										# Number of members of this object

		# Check for VSX objects ...
		my ($vs_mgmt, $vs_mgmt_obj);
		# (VSX objects are all listed in a separate table)
		my $vs_table = 'vs_slot_objects';
		if ($type =~ /VSX/) {
   			# Find Management of Virtual Gateway ...
   			$vs_mgmt_obj = GetKey($vs_table, $name);			# Ref to management object of VSX device
   			$vs_mgmt     = $name;
		}

		#----------------------------------------------------------------
		#---                  CHECKPOINT to HTML                      ---
		#----------------------------------------------------------------
		# Determine if 'normal'firewall software is installed ...
		my $FW_Installed = ((GetKey($obj, 'firewall') eq 'installed') && 
							($type !~ /Embedded/) && 
							(GetKey($obj, 'location') ne 'external') &&
							(GetKey($obj, 'junction') ne 'true'));
		# Determine if this is a 'normal' gateway with VPN installed and is locally managed ...
		my $VPN_Installed = (((GetKey($obj, 'VPN_1')    eq 'true') || (GetKey($obj, 'vpnddcate') eq 'true')) &&
							($type !~ /Embedded/) && (GetKey($obj, 'location') ne 'external'));
		
		# Set indentation level; this is done by setting the corresponding
		# variable in the template file ...
		if ($level==1) { 
			$tobj[$cnt]{MEMBER} = 1 
		} elsif ($level==2) { 
			$tobj[$cnt]{MEMBER_of_MEMBER} = 1 
		}
		
		$tobj[$cnt]{OBJ_ID}   = stripquotes($name);
		$tobj[$cnt]{OBJ_TYPE} = $type;
		$tobj[$cnt]{OBJ_NAME} = HTMLObjectList($Policy, $obj, $name);

		#------------------------------------------------
		# IP address of Check Point device (always shown
		# if available)
		#------------------------------------------------
		undef $val;
		if ($type !~ /Profile/ ) {
			if (GetKey($obj, 'ipaddr') =~ /0.0.0/) {
				$val = 'Dynamic Address'
			} elsif (GetKey($obj, 'ipaddr') ne "") {
				$val = GetKey($obj, 'ipaddr');
			}
		}
		$layout  = 'OBJ_'.$layout{'IP Address'};
		if ($val) {
			push @{$tobj[$cnt]{$layout}}, (
				{ TXT => [ { TEXT => html($val) } ] }
			);
		}

		#------------------------------------------------
		# General Properties ...
		#------------------------------------------------
		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'General Properties');
		$layout  = 'OBJ_'.$layout{'General Properties'};
		if ($section) {
			# --- Embedded Device ---
			# (there are no embedded devices anymore in NGX ...)
			if ($type eq 'Embedded Device') {
				push @{$tobj[$cnt]{$layout}}, (
					{ SF_BEG => [
						{ 	SECTION => html('General Properties'), 
							OBJ_NAME => $name, 
							STATE    => ($section eq $_OPEN) }
					] },
					{ SECTION => [
						{	CP2 => 1,
							OBJ_VAL1 => html('Type:'),
							OBJ_VAL2 => html('Nokia IP5x') }
					] },
					{ SECTION => [
						{	CP4 => 1,	
							OBJ_VAL1 => $CheckTick,
							OBJ_VAL2 => html('VPN-1 & FireWall-1 installed'),
							OBJ_VAL3 => html('Version:'), 
							OBJ_VAL4 => number(GetKey($obj, 'fwver')) }
					] },
					{ SECTION => [
						{	CP2 => 1,
							OBJ_VAL1 => html('License type:'),
							OBJ_VAL2 => number(GetKey($obj, 'embedded_lictype')) }
					] },
					{ SF_END => 1 }
				);

			} elsif ($type =~ /Profile/ ) {
				# No Check Point Products
			} elsif ($class =~ /interspect/ ) {
				# Special case (see below for Device Properties...)
			} elsif ($class =~ /connectra/ ) {
				# Special case (see below for Device Properties...)
			
			#------------------------------------------------
			# Sofaware - General Properties ...
			#------------------------------------------------
			} elsif ($class =~ /sofaware/ ) {
				push @{$tobj[$cnt]{$layout}}, (
					{ SF_BEG => [
						{	SECTION  => html('General Properties'), 	
							OBJ_NAME => $name, 
							STATE    => ($section eq $_OPEN) } 
					] },
					{ SECTION => [
						{	C2 => 2,
							OBJ_VAL1 => (GetKey($obj, 'sofaware_vpn') eq 'true')?$CheckTick:$Check,
							OBJ_VAL2 => html('VPN Enabled') },
					 	{	INACTIVE => (GetKey($obj, 'sofaware_vpn') eq 'false'),
					 		OBJ_VAL2 => ((GetKey($obj, 'sofaware_mode') eq 'SiteToSite')?$MarkDot:$Mark),
							OBJ_VAL3 => html('Connects as Site to Site Gateway') },
					 	{	INACTIVE => (GetKey($obj, 'sofaware_vpn') eq 'false'),
							OBJ_VAL2 => ((GetKey($obj, 'sofaware_mode') eq 'SR')?$MarkDot:$Mark),
							OBJ_VAL3 =>	html('Connects as Remote Access Client') },
					] }
				);
	
				%var = %{obj2html($Policy, GetKey($obj, 'profile:Table'), GetKey($obj, 'profile:Name'), $_HREF)};
				$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
				push @{$tobj[$cnt]{$layout}}, (
					{ SECTION => [
						{	CP2 => 1, 
							OBJ_VAL1 => html('Type:'), 
							OBJ_VAL2 => html(GetKey($obj, 'product_type:Name')) },
						 {	INACTIVE => (($version ne "NG") || (GetKey($obj, 'external') eq 'true')), 
						 	CP2 => 1, 
							OBJ_VAL1 => html('VPN-1 Edge Profile:'), 
							OBJ_VAL2 => $var }
					] },
					{ SECTION => [
						{	OBJ_VAL1 => (GetKey($obj, 'external') eq 'true')?$CheckTick:$Check,
							OBJ_VAL2 => html('Externally Managed Gateway') }
					] },
					{ SF_END => 1 }
				);

       		#------------------------------------------------
    		# VSX Gateway/Cluster - General Properties
       		#------------------------------------------------
    		} elsif ($type =~ /VSX/) {
    			push @{$tobj[$cnt]{$layout}}, (
    				{ SF_BEG => [
    					{ 	SECTION => html('General Properties'), 
    						OBJ_NAME => $name, 
    						STATE    => ($section eq $_OPEN) }
    				] },
    				{ SECTION => [
    					{	CP2 => 1,
    						OBJ_VAL1 => html('VSX Gateway Name:'),
    						OBJ_VAL2 => html($vs_mgmt) },
    					{	CP2 => 1,
    						OBJ_VAL1 => html('VSX Gateway Version:'),
    						OBJ_VAL2 => (GetKey($vs_mgmt_obj, 'vsxver') eq '210')?html("VSX NG AI"):html("VSX 2.0.1") },
    					{	INACTIVE => (GetKey($vs_mgmt_obj, 'AdminInfo:ClassName') !~ /cluster/),
    						CP2 => 1,
    						OBJ_VAL1 => html('Cluster Type:'),
    						OBJ_VAL2 => html($vsx_cluster_type{GetKey($vs_mgmt_obj, 'third_party_solution')}) },
    				] },
    				{ SF_END => 1 }
    			);
    
			#------------------------------------------------
			# Other gateways - General Properties
			#------------------------------------------------
			} elsif (GetKey($obj, 'cp_products_installed') eq 'true') {
				$var = '';
				if ($cpversion ge '6') {
					$var .= 'NGX R60';
				} elsif ($cpversion ge '5') {
					$var .= 'NG';
					if ($cpversion ge '5.0.5') { 
						$var .= ' with Application Intelligence - R55W';
					} elsif ($cpversion ge '5.0.4') { 
						$var .= ' with Application Intelligence';
					} elsif ($cpversion ge '5.0') {
						$var .= ' Feature Pack '.GetKey($obj, 'option_pack');
					}
				} else {
					# Catch all for unknown versions ...
					$var .= GetKey($obj, 'fwver');
				}

				push @{$tobj[$cnt]{$layout}}, (
					{ 	SF_BEG => [
						{	SECTION  => html('General Properties'), 
							OBJ_NAME => $name, 
							STATE    => ($section eq $_OPEN) }
					] },
					{	SECTION	=>	[
						{  	CP2 => 1,	
							OBJ_VAL1 => html('Version:'), 
							OBJ_VAL2 => html($var) },
						{ 	INACTIVE => ($version eq 'NG'),
							CP2 => 1,	
							OBJ_VAL1 => html('OS:'), 
							OBJ_VAL2 => html(stripquotes(GetKey($obj, 'os_info:osName')||"Unknown OS")) },
						{	INACTIVE => ($cpversion lt '5.0.4'),
							CP2 => 1,
							OBJ_VAL1 => html('Type:'),
							OBJ_VAL2 => ((GetKey($obj, 'cp_suite_type') eq 'pro')?'Check Point Enterprise/Pro':'Check Point Express') }
					] }
				);
	
				# Check Point products installed
				@var = ();
				if (GetKey($obj, 'firewall') eq 'installed') 		{ push @var, ($version eq 'NG')?'FireWall-1':'Firewall' }
				if (GetKey($obj, 'VPN_1')    eq 'true')				{ push @var, ($version eq 'NG')?'VPN-1 Pro':'VPN' }
				if (($version eq 'NG') && (GetKey($obj, 'vpnddcate') eq 'true'))
																	{ push @var, 'VPN-1 Net' }
				if (GetKey($obj, 'floodgate') eq 'installed')		{ push @var, ($version eq 'NG')?'Floodgate-1':'QoS' }
				if ((($class eq "gateway_cluster") || ($class eq "vsx_cluster_netobj")) && (GetKey($obj, 'CP_high_availability') eq 'true'))
																	{ push @var, 'ClusterXL' }
				if (GetKey($obj, 'policy_server') eq 'installed')	{ push @var, 'SecureClient Policy Server' }
				if (($version eq 'NG') && (GetKey($obj, 'SDS') eq 'installed'))
																	{ push @var, 'SecureClient Software Distribution Server' }
				if (GetKey($obj, 'management') eq 'true') {
					if (GetKey($obj, 'primary_management') eq 'true')	{ 
						push @var, ($version eq 'NG')?'Primary Management Station':'Primary SmartCenter Server';
					} else { 
						push @var, ($version eq 'NG')?'Secondary Management Station':'Secondary SmartCenter Server';
					}
				}

				# SVN foundation if normal Check Point host or gateway, or the routing object in the VSX config ...				
				if (($class =~ /ckp/) || ($class eq "vsx_netobj") || (GetKey($obj, 'junction') eq 'true')) 	
																	{ push @var, 'SVN Foundation' }
				if (GetKey($obj, 'log_server') eq 'true')			{ push @var, 'Log Server' }
				if (($version ne 'NG') && (GetKey($obj, 'sc_portal') eq 'true'))		
																	{ push @var, 'SmartPortal' }
				if (GetKey($obj, 'reporting_server') eq 'true')		{ push @var, ($version eq 'NG')?'SmartView Reporter':'Eventia Reporter' }
				if (GetKey($obj, 'abacus_server') eq 'true')		{ push @var, 'Eventia Analyzer Server' }
				if (GetKey($obj, 'event_analyzer') eq 'true')		{ push @var, 'Eventia Correlation Unit' }
				if (GetKey($obj, 'real_time_monitor') eq 'true')	{ push @var, 'SmartView Monitor' }
				if (GetKey($obj, 'UA_gateway') eq 'true')			{ push @var, 'UserAuthority Server' }
				if (GetKey($obj, 'UA_WebAccess') eq 'true')			{ push @var, 'UserAuthority WebAccess' }
				if (($version eq 'NG') && (GetKey($obj, 'Everest') eq 'true'))
																	{ push @var, 'Firewall-1 GX' }

				my @section;
				if (@var) {
					foreach $var (@var) {
						push @section, { OBJ_VAL1 => $CheckTick, OBJ_VAL2 => $var };
					}
				} else {
					push @section, { OBJ_VAL1 => $None };
				}
	
				push @{$tobj[$cnt]{$layout}}, ( 
					{	SECTION_BEGIN => 'Check Point Products' }, 
					{	SECTION => \@section },
					{	SECTION_END => 1 },
					{	SECTION => [
						{	INACTIVE => (($class ne "vs_cluster_netobj") && ($class ne "vsx_cluster_netobj")),
							OBJ_VAL1 => (GetKey('properties','firewall_properties:enforce_install_on_all_cluster_objects') eq "true")?$CheckTick:$Check,
							OBJ_VAL2 => html("Install configuration on all members, if fails, do not install at all") }
					] }, 
					{	SF_END => 1 }
				);
			}

		} # End of General Properties ...

		#------------------------------------------------
		# Device Properties ...
		# (special version of General Properties)
		#------------------------------------------------
		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'General Properties');
		$layout  = 'OBJ_'.$layout{'General Properties'};
		if ($section) {
			# Interspect - Device Properties ...
			if ($class =~ /interspect/ ) {
				push @{$tobj[$cnt]{$layout}}, (
					{ 	SF_BEG => [
						{	SECTION  => html('Device Properties'), 
							OBJ_NAME => $name, 
							STATE    => ($section eq $_OPEN) }
					] },
					{	SECTION	=>	[
						{  	CP2 => 1,	
							OBJ_VAL1 => html('Version:'), 
							OBJ_VAL2 => $version{GetKey($obj, 'version')} },
					] },
    				{ SF_END => 1 }
				);

			# Connectra - Device Properties ...
			} elsif ($class =~ /connectra/ ) {
				push @{$tobj[$cnt]{$layout}}, (
					{ 	SF_BEG => [
						{	SECTION  => html('Device Properties'), 
							OBJ_NAME => $name, 
							STATE    => ($section eq $_OPEN) }
					] },
					{	SECTION	=>	[
						{  	CP2 => 1,	
							OBJ_VAL1 => html('Port:'), 
							OBJ_VAL2 => html(GetKey($obj, 'port')) },
					] },
    				{ SF_END => 1 }
				);
    		}
    	} # End of Device Properties ...

   		#------------------------------------------------
		# Cluster Members ...
   		#------------------------------------------------
   		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'Cluster Members');
   		$layout  = 'OBJ_'.$layout{'Cluster Members'};
   		if ($section) {
	   		#------------------------------------------------
			# VSX Cluster Members ...
   			#------------------------------------------------
			if ($type =~ /VSX/) {
				if ($type =~ /Cluster/) {
	    			my @section;
					foreach $var (GetMembers(GetKey($vs_mgmt_obj, 'cluster_members'))) {
						my $pat = $name."_(.*)_MVS";
						$_ = GetKey($var, 'Name');
						if (/$pat/) { $val = $1 }
						
	    				push @section, {
	    					CLASS => 'cpvalue2',
	    					TEXT  => $Spacer.html($val),
	    				};
	    			}
					push @{$tobj[$cnt]{$layout}}, (
						{ SF_BEG => [
							{ 	SECTION => html('Cluster Members'), 
								OBJ_NAME => $name, 
								STATE    => ($section eq $_OPEN) }
						] },
						{ SECTION => [
							{	OBJ_VAL1 => html('VSX Cluster Members names:') }
						] },
						{ TXT => \@section },
						{ TXT => [
							{ 	CLASS => 'note_xspc',
								TEXT  => html('Private network that will be used for internal communications;').
										 html('between VSX Cluster Members:') }
						] },
						{ SECTION => [
							{ 	CP2 => 1,
								OBJ_VAL1 => html('Network Address:'),
								OBJ_VAL2 => html(GetKey($vs_mgmt_obj, 'cluster_private_network:ipaddr')) },
							{	CP2 => 1,
								OBJ_VAL1 => html('Net Mask:'),
								OBJ_VAL2 => html(GetKey($vs_mgmt_obj, 'cluster_private_network:netmask')) },
						] },
						{ SF_END => 1 }
					);
				}
	
				#------------------------------------------------
				# Find and process VSX subordinate devices ...
				#------------------------------------------------
				my ($vs_name, $vs_obj, @vs_obj);

				# Output VS_Mgmt object to HTML ...
				#$members += cpobject2html(GetKey(GetKey($vs_obj, 'Table'), GetKey($vs_obj, 'Name')), "Check Point VS Cluster Properties", GetKey($vs_obj, 'Name'), ($cnt+$members+1), $level+1);

				# Find all objects under CONTROL of this mgmt object ...
				foreach $vs_name (keys %{GetKey($vs_table)}) {
					$vs_obj = GetKey($vs_table, $vs_name);
					if (($vs_name ne $vs_mgmt) && (GetKey($vs_obj, 'vsx_gateway:Name') eq $vs_mgmt)){
						push @vs_obj, $vs_name;
					}
				}
				# Sort the objects alphabetically and iterate over them ...
				foreach $vs_name (sort @vs_obj) {
					$vs_obj = GetKey($CPtable, $vs_name);
					if (GetKey($vs_obj, 'AdminInfo:ClassName') =~ /cluster/ ) {
						$members += cpobject2html(GetKey($CPtable, $vs_name), "Check Point Virtual System/Router Cluster", $vs_name, ($cnt+$members+1), $level+1);
					} else {
						$members += cpobject2html(GetKey($CPtable, $vs_name), "Check Point Virtual System/Router", $vs_name, ($cnt+$members+1), $level+1);
					}
				} 
			# End of VSX Cluster Members ...
			    
    		#------------------------------------------------
    		# Gateway Cluster Members ...
    		#------------------------------------------------
    		} elsif (($type =~ /Cluster/) && ($type !~ /Member/)) {
    			$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'Cluster Members');
    			$layout  = 'OBJ_'.$layout{'Cluster Members'};
    			if ($section) {
    				my @section;
    				# Find Members ...
    				@var = GetMembers(GetKey($obj, 'cluster_members'));
    				if (@var) {
    					push @section, {	
    						HEAD => 1, 
    						OBJ_VAL1 => html('Name'), 
    						OBJ_VAL2 => html('IP address'), 
    					};
    
    					foreach $var (@var) {
    						%var = %{obj2html($Policy, GetKey($var, 'Table'), GetKey($var, 'Name'), $_HREF)};
    						$val = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    						push @section, { 
    							OBJ_VAL1 => $val,
    							OBJ_VAL2 => GetKey(GetKey(GetKey($var, 'Table'), GetKey($var, 'Name')), 'ipaddr')
    						};
    
    						# Add cluster member to table ...
    						$members += cpobject2html(GetKey(GetKey($var, 'Table'), GetKey($var, 'Name')), "Check Point Cluster Member", GetKey($var, 'Name'), ($cnt+$members+1), $level+1);
    					}
    				} else {
    					push @section, { OBJ_VAL1 => $None };
    				}
    				push @{$tobj[$cnt]{$layout}}, (
    					{ 	SF_BEG => [{	SECTION  => html('Cluster Members'), 
    										OBJ_NAME => $name, 
    										STATE    => ($section eq $_OPEN) }],
    						TABLE  =>	\@section,
    						SF_END => 1
    					}
    				);
    			} # End of Gateway Cluster Members ...
    		} 
		} # End of Cluster Members ...

	
   		#------------------------------------------------
		# VSX System Interfaces ...
   		#------------------------------------------------
		if ($type =~ /VSX/) {
    		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'System Interfaces');
    		$layout  = 'OBJ_'.$layout{'System Interfaces'};
    		if ($section) {
    			my @section;
   				push @section, {
					HEAD => 1,
					OBJ_VAL1 => html('Interfaces for Virtual Systems'),
					OBJ_VAL2 => html('Allow VLAN')
				};
				foreach $var (GetMembers(GetKey($vs_mgmt_obj, 'physical_interfaces'))) {
    				push @section, {
    					OBJ_VAL1 => ((GetKey($var,'available') eq 'true')?$CheckTick:$Check)." ".GetKey($var, 'name'),
    					OBJ_VAL2 => ((GetKey($var,'vlan_support') eq 'true')?$CheckTick:$Check)
    				};
    			}
				push @{$tobj[$cnt]{$layout}}, (
					{ SF_BEG => [
						{ 	SECTION => html('System Interfaces'), 
							OBJ_NAME => $name, 
							STATE    => ($section eq $_OPEN) }
					] },
					{ TABLE => \@section },
					{ SF_END => 1 }
				);
    		}
		} # End of VSX System Interfaces ...
		
		#------------------------------------------------
		# Content Filtering (Sofaware only) ...
		#------------------------------------------------
		if (GetKey($obj, 'type') eq 'sofaware_gateway') {
			$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'Content Filtering');
			$layout  = 'OBJ_'.$layout{'Content Filtering'};
			if ($section) {
    			push @{$tobj[$cnt]{$layout}}, (
    				{ SF_BEG => [
    					{	SECTION  => html('Content Filtering'), 
    						OBJ_NAME => $name, 
    						STATE    => ($section eq $_OPEN) }
    				] },
    				{ SECTION => [
    					{	OBJ_VAL1 => (GetKey($obj, 'enable_sofaware_ufp') eq "true")?$CheckTick:$Check,
    						OBJ_VAL2 => html("Use UFP") },
    					{	OBJ_VAL1 => (GetKey($obj, 'enable_sofaware_cvp') eq "true")?$CheckTick:$Check,
    						OBJ_VAL2 => html("Use CVP") },
    				] },
    				{ SECTION => [
    					{	OBJ_VAL1 => html('Note:'),
    						OBJ_VAL2 => html('Global UFP/CVP configuration will be done via the "VPN-1;').
    									html('Edge/Embedded Gateway" tab under the Global Properties.') },
    				] },
    				{ SF_END => 1 }
    			);
			}
		}

		#------------------------------------------------
		# 3rd Party Configuration ...
		#------------------------------------------------
		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', '3rd Party Configuration');
		$layout  = 'OBJ_'.$layout{'3rd Party Configuration'};
		if ($section) {
			# Determine 3rd Party Configuration section should be displayed ...
			if ((($class eq "gateway_cluster") || ($class eq "vsx_cluster_netobj")) && 
				 (GetKey($obj, 'CP_high_availability') eq 'false')) {

    			push @{$tobj[$cnt]{$layout}}, (
    				{ SF_BEG => [
    					{	SECTION  => html('3rd Party Configuration'), 
    						OBJ_NAME => $name, 
    						STATE    => ($section eq $_OPEN) }
    				] },
    				{ SECTION_BEGIN => html('Cluster operating mode:') },
    				{ SECTION => [
    					{	OBJ_VAL1 => (GetKey($obj, 'availability_mode') eq 'HighAvailability')?$MarkDot:$Mark, 
    						OBJ_VAL2 => html("High Availability") },
    					{	OBJ_VAL1 => (GetKey($obj, 'availability_mode') eq 'LoadSharing')?$MarkDot:$Mark, 
    						OBJ_VAL2 => html("Load Sharing") }
    				] },
    				{ SECTION => [
    					{	CP2 => 1,
    						OBJ_VAL1 => html("3rd Party Solution: "),
    						OBJ_VAL2 => html($third_party_solution{GetKey($obj, 'third_party_solution')}) }
    				] },
    				{ SECTION_END => 1 },
    				{ SECTION => [
    					{	OBJ_VAL1 => (GetKey($obj, 'use_sync') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html("Use State Synchronization") }
    				] },
    				{ TXT => [
    					{	CLASS => 'note_xspc',
    						TEXT => html("See the OPSEC cluster solution's manual for the correct;").
    								html("settings of these check boxes.") }
    				] },
    				{ SECTION => [
    					{	OBJ_VAL1 => (GetKey($obj, 'use_limited_flushnack') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html("Support non-sticky connections") },
    					{	OBJ_VAL1 => (GetKey($obj, 'perform_cluster_hide_fold') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html("Hide Cluster Members' outgoing traffice behind the Cluster's IP Address") },
    					{	OBJ_VAL1 => (GetKey($obj, 'perform_cluster_fold') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html("Forward Cluster's incoming traffice to Cluster Members' IP Addresses") },
    				] },
    				{ SF_END => 1 }
    			);
    		}
    	}

		#------------------------------------------------
		# ClusterXL ...
		#------------------------------------------------
		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'ClusterXL');
		$layout  = 'OBJ_'.$layout{'ClusterXL'};
		if ($section) {
			# Determine ClusterXL section should be displayed ...
			if ((($class eq "gateway_cluster") || ($class eq "vsx_cluster_netobj")) && 
				 (GetKey($obj, 'CP_high_availability') eq 'true')) {

    			push @{$tobj[$cnt]{$layout}}, (
    				{ SF_BEG => [
    					{	SECTION  => html('ClusterXL'), 
    									OBJ_NAME => $name, 
    									STATE    => ($section eq $_OPEN) }
    				] },
    				{ SECTION_BEGIN => html('Cluster Mode') },
    				{ SECTION => [
    					{	OBJ_VAL1 => (GetKey($obj, 'availability_mode') eq 'HighAvailability')?$MarkDot:$Mark, 
    						OBJ_VAL2 => html("High Availability") }
    				] }
    			);
    			if (GetKey($obj, 'availability_mode') eq 'HighAvailability') {
	    			push @{$tobj[$cnt]{$layout}}, (
        				{ SECTION => [
        					{	OBJ_VAL1 => $Spacer,
        						OBJ_VAL2 => (GetKey($obj, 'HA_new') eq 'true')?$MarkDot:$Mark, 
        						OBJ_VAL3 => html("New Mode") },
        					{	OBJ_VAL1 => $Spacer,
        						OBJ_VAL2 => (GetKey($obj, 'HA_new') eq 'false')?$MarkDot:$Mark, 
        						OBJ_VAL3 => html("Legacy Mode") },
        				] }
        			);
        		}
    			push @{$tobj[$cnt]{$layout}}, (
    				{ SECTION => [
    					{	OBJ_VAL1 => (GetKey($obj, 'availability_mode') eq 'LoadSharing')?$MarkDot:$Mark, 
    						OBJ_VAL2 => html("Load Sharing") }
    				] }
    			);
    			if (GetKey($obj, 'availability_mode') eq 'LoadSharing') {
	    			push @{$tobj[$cnt]{$layout}}, (
        				{ SECTION => [
        					{	OBJ_VAL1 => $Spacer,
        						OBJ_VAL2 => (GetKey($obj, 'cluster_LS_mode') eq 'Multicast')?$MarkDot:$Mark, 
        						OBJ_VAL3 => html("Multicast Mode") },
        					{	OBJ_VAL1 => $Spacer,
        						OBJ_VAL2 => (GetKey($obj, 'cluster_LS_mode') eq 'Pivot')?$MarkDot:$Mark, 
        						OBJ_VAL3 => html("Unicast Mode") },
        				] },

        				# Advanced settings for LoadSharing ...
						{ SF_BEG => [
							{	SECTION  => html('Advanced...'),
								BUTTON   => 1,
								OBJ_NAME => $name,
								STATE	 => 0 }
						] },
						{ SECTION_BEGIN => html("Use sharing method based on:") },
						{ SECTION => [
							{	OBJ_VAL1 => (GetKey($obj, 'LS_decision_function') eq 'default')?$MarkDot:$Mark,
								OBJ_VAL2 => html("IPs, Ports, SPIs (default)") },
							{	OBJ_VAL1 => (GetKey($obj, 'LS_decision_function') eq 'IPSEC_IPBased')?$MarkDot:$Mark,
								OBJ_VAL2 => html("IPs, Ports") },
							{	OBJ_VAL1 => (GetKey($obj, 'LS_decision_function') eq 'IPBased')?$MarkDot:$Mark,
								OBJ_VAL2 => html("IPs") },
						  ] },
						{ SECTION_END => 1 },
						{ SECTION => [
    						{ 	OBJ_VAL1 => (GetKey($obj, 'use_sdf') eq 'true')?$CheckTick:$Check, 
    							OBJ_VAL2 => html("Use Sticky Decision Function") },
    					] },
    					{ SF_END => 1 },
        			);
        		}

				# Close Cluster Mode section ...
    			push @{$tobj[$cnt]{$layout}}, { SECTION_END => 1 };

    			if (GetKey($obj, 'availability_mode') eq 'HighAvailability') {
	    			push @{$tobj[$cnt]{$layout}}, (
						{ SECTION => [
    						{ 	OBJ_VAL1 => (GetKey($obj, 'use_sync') eq 'true')?$CheckTick:$Check, 
    							OBJ_VAL2 => html("Use State Synchronization") },
    					] }
    				);
    				
    				# Upon gateway recovery section ...
	    			push @{$tobj[$cnt]{$layout}}, (
    					{ SECTION_BEGIN => html('Upon Gateway recovery') },
    					{ SECTION => [
    						{	OBJ_VAL1 => (GetKey($obj, 'HA_mode') eq 'ActiveUp')?$MarkDot:$Mark, 
    							OBJ_VAL2 => html("Maintain current active Gateway") },
    						{	OBJ_VAL1 => (GetKey($obj, 'HA_mode') eq 'HigherUp')?$MarkDot:$Mark, 
    							OBJ_VAL2 => html("Switch to higher priority Gateway") },
    					] },
    					{ SECTION_END => 1 },
    					{ SECTION => [
    						{	CP2 => 1,
    							OBJ_VAL1 => html("Fail-Over Tracking: "),
    							OBJ_VAL2 => html($Track{GetKey($obj, 'fail_over_track')}) }
    					] },
    				);
    				
    			}

				# Close ClusterXL section ...
    			push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };
    				

    		}
		}
		
		#------------------------------------------------
		# TOPOLOGY ...
		#------------------------------------------------
		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'Topology');
		$layout  = 'OBJ_'.$layout{'Topology'};
		if ($section) {
			push @{$tobj[$cnt]{$layout}}, Topology($Policy, $obj, $name, $type, $section);
		}
		
		#------------------------------------------------
		# SYNCHRONIZATION ...
		#------------------------------------------------
		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'Synchronization');
		$layout  = 'OBJ_'.$layout{'Synchronization'};
		if ($section) {
			# Determine if Sync section should be displayed ...
			if (($class eq "gateway_cluster") || ($class eq "vsx_cluster_netobj")) {
				# Find sync networks ...
				@var = GetMembers(GetKey($obj, 'synchronization_networks'));
				if (@var) {
					push @{$tobj[$cnt]{$layout}}, (
        				{ SF_BEG => [
        					{	SECTION  => html('Synchronization'), 
        									OBJ_NAME => $name, 
        									STATE    => ($section eq $_OPEN) }
        				] },
        			);
        			my @section;
        			push @section, {
    					HEAD => 1, 
    					OBJ_VAL1 => html('Network Name'), 
    					OBJ_VAL2 => html('IP Address'),
    					OBJ_VAL3 => html('Network Mask'),
    				};
    				foreach $var (@var) {
        				push @section, {
    						OBJ_VAL1 => html(stripquotes(GetKey($var, 'description'))), 
    						OBJ_VAL2 => GetKey($var, 'ipaddr'),
    						OBJ_VAL3 => GetKey($var, 'netmask'),
    					};
    				}
					push @{$tobj[$cnt]{$layout}}, (
        				{ SECTION_BEGIN => html('Synchronization networks:') },
        				{ TABLE => \@section, C3 => 1 },
        				{ TXT => [
        					{ TEXT => html("Make sure the above networks are safe."),
        					  CLASS => 'note_xspc' }
        				] },
        				{ SECTION_END => 1 }
        			);

					push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };
					
        		}
        	}
		}
		
		#------------------------------------------------
		# NAT ...
		#------------------------------------------------
		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'NAT');
		$layout  = 'OBJ_'.$layout{'NAT'};
		if ($section) {
			push @{$tobj[$cnt]{$layout}}, NAT($Policy, $obj, $name, $type, $section);
		}

		#------------------------------------------------
		# VPN ...
		#------------------------------------------------
		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'VPN');
		$layout  = 'OBJ_'.$layout{'VPN'};
		if ($section) {
			push @{$tobj[$cnt]{$layout}}, VPN($Policy, $obj, $name, $type, $section);
		}
		
		#------------------------------------------------
		# Remote Access ...
		#------------------------------------------------
		if ($VPN_Installed && (Version($obj) ge '5.01')) {
			# CheckPoint VPN is installed and version > NG (no FP) ...
			$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'Remote Access');
			$layout  = 'OBJ_'.$layout{'Remote Access'};
			if ($section) {
				push @{$tobj[$cnt]{$layout}}, (
					{ 	SF_BEG => [
						{ 	SECTION  => html('Remote Access'), 
							OBJ_NAME => $name, 
							STATE    => ($section eq $_OPEN) 
						}]
					}
				);
	
				if (Version($obj) ge '5.03') {
    				# L2TP Support
    				push @{$tobj[$cnt]{$layout}}, (
    					{ SECTION_BEGIN => html('L2TP Support') },
    					{ SECTION	=> [
    						{	C2 => 2,
    							OBJ_VAL1 => (GetKey($obj, 'support_l2tp') eq 'true')?$CheckTick:$Check,
    							OBJ_VAL2 => html('Support L2TP (relevant only when Office Mode is active)') },
    						{	INACTIVE => (GetKey($obj, 'support_l2tp') eq 'false'),
    							CP3 => 1,
    							OBJ_VAL1 => $Spacer,
    							OBJ_VAL2 => html('Authentication Method:'),
    							OBJ_VAL3 => (GetKey($obj, 'l2tp_auth_method') eq 'MD5')?html('MD5-Challenge'):html('Smart Card or other Certificate') },
    						{	INACTIVE => (GetKey($obj, 'support_l2tp') eq 'false'),
    							CP3 => 1,
    							OBJ_VAL1 => $Spacer,
    							OBJ_VAL2 => html('Use this certificate:'),
    							OBJ_VAL3 => html(GetKey($obj, 'use_cert_for_l2tp')) }
    					] },
    					{	SECTION_END => 1 }
    				);
				}
				
				if (Version($obj) ge '5.04') {
					# Hub Mode configuration ...
					push @{$tobj[$cnt]{$layout}}, (
						{ SECTION_BEGIN => html('Hub Mode Configuration') },
						{ SECTION => [
							{	OBJ_VAL1 => (GetKey($obj, 'allow_VPN_routing_from_SR') eq 'true')?$CheckTick:$Check,
								OBJ_VAL2 => html('Allow SecureClient to route traffic through this gateway') }
						] },
						{ SECTION_END => 1 }
					);
				}
	
				# NAT traversal ...
				%var = %{obj2html($Policy, GetKey($obj, 'isakmp.udpencapsulation:resource:Table'), GetKey($obj, 'isakmp.udpencapsulation:resource:Name'), $_HREF)};
				$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
				push @{$tobj[$cnt]{$layout}}, (
					{ SECTION_BEGIN => html('NAT traversal') },
					{ SECTION => [
						{	C2 => 2,
							OBJ_VAL1 => (GetKey($obj, 'isakmp.udpencapsulation:active') eq 'true')?$CheckTick:$Check,
							OBJ_VAL2 => html('Support NAT traversal mechanism (UDP encapsulation)') },
						{	INACTIVE => (GetKey($obj, 'isakmp.udpencapsulation:active') eq 'false'),
							CP3 => 1,
							OBJ_VAL1 => $Spacer,
							OBJ_VAL2 => html('Allocated port:'),
							OBJ_VAL3 => $var }
					] },
					{ SECTION_END => 1 }
				);
	
				if (Version($obj) ge '5.04') {
					# Visitor mode configuration ...
					%var = %{obj2html($Policy, GetKey($obj, 'tcpt:resource:Table'), GetKey($obj, 'tcpt:resource:Name'), $_HREF)};
					$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
					push @{$tobj[$cnt]{$layout}}, (
						{ SECTION_BEGIN => html('Visitor Mode configuration') },
						{ SECTION => [
							{	C2 => 2,
								OBJ_VAL1 => (GetKey($obj, 'tcpt:active') eq 'true')?$CheckTick:$Check,
								OBJ_VAL2 => html('Support Visitor Mode') },
							{	INACTIVE => (GetKey($obj, 'tcpt:active') eq 'false'),
								CP3 => 1,
								OBJ_VAL1 => $Spacer,
								OBJ_VAL2 => html('Allocated port:'),
								OBJ_VAL3 => $var },
							{	INACTIVE => (GetKey($obj, 'tcpt:active') eq 'false'),
								CP3 => 1,
								OBJ_VAL1 => $Spacer,
								OBJ_VAL2 => html('Allocated IP Address:'),
								OBJ_VAL3 => html(stripquotes(GetKey($obj, 'tcpt:interface'))) }
						] },
						{ SECTION_END => 1 }
					);
				}
			}

			#------------------------------------------------
			# Remote Access - Office mode ...
			#------------------------------------------------
			push @{$tobj[$cnt]{$layout}}, (
				{ SF_BEG => [
					{	SECTION  => html('Office Mode'), 
						SUB      => 1,
						OBJ_NAME => $name, 
						STATE    => 0 }
				] }
			);
			
   			%var = %{obj2html($Policy, GetKey($obj, 'ip_assignment_group:Table'), GetKey($obj, 'ip_assignment_group:Name'), $_HREF)};
   			$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
			push @{$tobj[$cnt]{$layout}}, (
				{ SECTION => [
					{	OBJ_VAL1 => (GetKey($obj, 'ip_assignment_offer') eq 'never')?$MarkDot:$Mark,
						OBJ_VAL2 => html('Do not offer Office Mode') },
					{	INACTIVE => (GetKey($obj, 'ip_assignment_offer') eq 'per_group'),
					 	CP3 => 1,
					 	OBJ_VAL1 => (GetKey($obj, 'ip_assignment_offer') eq 'per_group')?$MarkDot:$Mark,
					 	OBJ_VAL2 => html('Offer Office Mode to group') },
					{	INACTIVE => (GetKey($obj, 'ip_assignment_offer') ne 'per_group'),
					 	CP3 => 1,
					 	OBJ_VAL1 => (GetKey($obj, 'ip_assignment_offer') eq 'per_group')?$MarkDot:$Mark,
					 	OBJ_VAL2 => html('Offer Office Mode to group:'),
					 	OBJ_VAL3 => $var },
					{	OBJ_VAL1 => (GetKey($obj, 'ip_assignment_offer') eq 'always')?$MarkDot:$Mark,
						OBJ_VAL2 => html('Allow Office Mode to all users') }
				] }
			);

			# Office mode Method ...
			if (GetKey($obj, 'ip_assignment_offer') ne 'never') {

				push @{$tobj[$cnt]{$layout}}, (
					{ SECTION_BEGIN => html('Office Mode Method') },
				);

				if ($version ne "NG") {
					push @{$tobj[$cnt]{$layout}}, (
						{ TXT => [
							{	TEXT => html("Allocate IP address by sequentially trying the checked methods, until success:"),
								CLASS => 'note' }
						] },
						{ SECTION => [
							{	OBJ_VAL1 => (GetKey($obj, 'ip_assignment_settings'))?$CheckTick:$Check,
								OBJ_VAL2 => html('From ipassignment.conf located in $FWDIR/conf - always tried first') },
							{	OBJ_VAL1 => (GetKey($obj, 'om_method_radius') eq 'true')?$CheckTick:$Check,
								OBJ_VAL2 => html("From the RADIUS server used to authenticate the user") },
							{	OBJ_VAL1 => (GetKey($obj, 'om_method_standard') eq 'true')?$CheckTick:$Check,
								OBJ_VAL2 => (GetKey($obj, 'om_method_standard') eq 'true')?	html("Using one of the following methods:"):
																							html("Using Manual (IP pool) or Automatic (DHCP)") },
						] }
					);
				}

				if ((GetKey($obj, 'om_method_standard') eq 'true') || ($version eq "NG")) {					
    				if (GetKey($obj, 'om_method') eq 'ippool') {
    					%var  = %{obj2html($Policy, GetKey($obj, 'om_ippool:Table'), GetKey($obj, 'om_ippool:Name'), $_HREF)};
    					$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    					push @{$tobj[$cnt]{$layout}}, (
    						{ SECTION => [
    							{	OBJ_VAL1 => ($version ne "NG")?$Spacer:"",
    								OBJ_VAL2 => $MarkDot,
    								OBJ_VAL3 => html('Manual (using IP pool)') },
    							{	OBJ_VAL2 => $Spacer,
    								OBJ_VAL3 => html('Allocate IP from network:'),
    								OBJ_VAL4 => $var },
    							{	OBJ_VAL2 => $Mark,
    								OBJ_VAL3 => html('Automatic (using DHCP)') }
    						] },
    					);
    				} else {
    					%var  = %{obj2html($Policy, GetKey($obj, 'om_dhcp_server:Table'), GetKey($obj, 'om_dhcp_server:Name'), $_HREF)};
    					$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    					push @{$tobj[$cnt]{$layout}}, (
    						{ SECTION => [
    							{	OBJ_VAL1 => ($version ne "NG")?$Spacer:"",
    								OBJ_VAL2 => $Mark,
    								OBJ_VAL3 => html('Manual (using IP pool)') },
    							{	OBJ_VAL2 => $MarkDot,
    								OBJ_VAL3 => html('Automatic (using DHCP)') },
    							{	OBJ_VAL2 => $Spacer,
    								OBJ_VAL3 => html('User specific DHCP server:'),
    								OBJ_VAL4 => $var },
    							{	CP4 => 1,
    								OBJ_VAL2 => $Spacer,
    								OBJ_VAL3 => html('Virtual IP address for DHCP;server replies:'),
    								OBJ_VAL4 => GetKey($obj, 'om_giaddr') },
    							{	CP4 => 1,
    								OBJ_VAL2 => $Spacer,
    								OBJ_VAL3 => html('MAC address for DHCP allocation:'),
    								OBJ_VAL4 => $MacCalc{GetKey($obj, 'om_mac_calc')} }
    						] },
    					);
    				}
    			}

				push @{$tobj[$cnt]{$layout}}, { SECTION_END => 1 };

				push @{$tobj[$cnt]{$layout}}, (
    				{ SF_BEG => [
    					{	SECTION  => html('Optional Parameters...'),
    						BUTTON   => 1,
    						OBJ_NAME => $name,
    						STATE	 => 0 }
    					] },
    				{ TXT => [
    					{	TEXT => html("This configuration applies to all Office Mode methods except;").
    								html("Automatic (using DHCP) and ipassignment.conf entries which;").
    								html("contain this data."),
    						CLASS => 'note' }
    				] },
    			);
    			
				# DNS Servers
				my ($dns1, $dns2, $dns3);
				if (GetKey($obj, 'om_dns_prim')) {
					%var  = %{obj2html($Policy, GetKey($obj, 'om_dns_prim:Table'), GetKey($obj, 'om_dns_prim:Name'), $_HREF)};
					$dns1 = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
				}
				if (GetKey($obj, 'om_dns_back1')) {
					%var  = %{obj2html($Policy, GetKey($obj, 'om_dns_back1:Table'), GetKey($obj, 'om_dns_back1:Name'), $_HREF)};
					$dns2 = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
				}
				if (GetKey($obj, 'om_dns_back2')) {
					%var  = %{obj2html($Policy, GetKey($obj, 'om_dns_back2:Table'), GetKey($obj, 'om_dns_back2:Name'), $_HREF)};
					$dns3 = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
				}
				push @{$tobj[$cnt]{$layout}}, (
    				{ SECTION_BEGIN => html("DNS servers") },
    				{ SECTION => [
    					{ 	OBJ_VAL1 => (GetKey($obj, 'om_enable_dns_prim') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html("Primary:"),
    						OBJ_VAL3 => $dns1 },
    					{ 	OBJ_VAL1 => (GetKey($obj, 'om_enable_dns_back1') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html("First backup:"),
    						OBJ_VAL3 => $dns2 },
    					{ 	INACTIVE => (GetKey($obj, 'om_enable_dns_back1') eq 'false'),
    					 	OBJ_VAL1 => (GetKey($obj, 'om_enable_dns_back2') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html("Second backup:"),
    						OBJ_VAL3 => $dns3 }
    				] },
    				{ SECTION_END => 1 }
    			);
    			undef $dns1; undef $dns2; undef $dns3;
    			
				# WINS Servers
				my ($wins1, $wins2, $wins3);
				if (GetKey($obj, 'om_wins_prim')) {
					%var  = %{obj2html($Policy, GetKey($obj, 'om_wins_prim:Table'), GetKey($obj, 'om_wins_prim:Name'), $_HREF)};
					$wins1 = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
				}
				if (GetKey($obj, 'om_wins_back1')) {
					%var  = %{obj2html($Policy, GetKey($obj, 'om_wins_back1:Table'), GetKey($obj, 'om_wins_back1:Name'), $_HREF)};
					$wins2 = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
				}
				if (GetKey($obj, 'om_wins_back2')) {
					%var  = %{obj2html($Policy, GetKey($obj, 'om_wins_back2:Table'), GetKey($obj, 'om_wins_back2:Name'), $_HREF)};
					$wins3 = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
				}
				push @{$tobj[$cnt]{$layout}}, (
    				{ SECTION_BEGIN => html("WINS servers") },
    				{ SECTION => [
    					{ 	OBJ_VAL1 => (GetKey($obj, 'om_enable_wins_prim') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html("Primary:"),
    						OBJ_VAL3 => $wins1 },
    					{ 	OBJ_VAL1 => (GetKey($obj, 'om_enable_wins_back1') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html("First backup:"),
    						OBJ_VAL3 => $wins2 },
    					{ 	INACTIVE => (GetKey($obj, 'om_enable_wins_back1') eq 'false'),
    						OBJ_VAL1 => (GetKey($obj, 'om_enable_wins_back2') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html("Second backup:"),
    						OBJ_VAL3 => $wins3 },
    					{	C1=>2, CP2 => 1,
    						OBJ_VAL1 => html("Domain name:"),
    						OBJ_VAL2 => GetKey($obj, 'om_domain_name') }
    				] },
    				{ SECTION_END => 1 }
    			);
    			undef $wins1; undef $wins2; undef $wins3;
    						
				# IP Lease duration ...
				push @{$tobj[$cnt]{$layout}}, (
					{ SECTION_BEGIN => html('IP Lease Duration') },
					{ SECTION => [
						{	CP2 => 1,
							OBJ_VAL1 => html('IP lease duration:'), 
							OBJ_VAL2 => number(GetKey($obj, 'om_lease_time')),
							OBJ_VAL3 => html('minutes') }
					] },
					{ SECTION_END => 1 },
					{ SF_END => 1 }
    			);

				if (Version($obj) ge '5.04') {
					# Multiple Interfaces ...
					push @{$tobj[$cnt]{$layout}}, (
						{ 	SECTION_BEGIN => 	html('Multiple Interfaces') },
						{	SECTION	=>	[{	OBJ_VAL1 => (GetKey($obj, 'om_enable_with_multiple_IF') eq 'true')?$CheckTick:$Check,
										OBJ_VAL2 => html('Support connectivity enhancement for gateways with;multiple external interfaces') }] },
						{	SECTION_END => 1 }
					);
				}

				if (Version($obj) ge '5.04') {
    				# Anti spoofing ...
    				%var  = %{obj2html($Policy, GetKey($obj, 'om_additional_ip_for_anti_spoofing:Table'), GetKey($obj, 'om_additional_ip_for_anti_spoofing:Name'), $_HREF)};
    				$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    				push @{$tobj[$cnt]{$layout}}, (
    					{ 	SECTION_BEGIN => 	html('Anti-Spoofing') },
    					{	SECTION	=>	[{	OBJ_VAL1 => (GetKey($obj, 'om_perform_antispoofing') eq 'true')?$CheckTick:$Check,
    									OBJ_VAL2 => html('Perform Anti-Spoofing on Office Mode addresses') }] },
    					{	SECTION	=>	[{	INACTIVE => (GetKey($obj, 'om_perform_antispoofing') eq 'false'),
    								 	OBJ_VAL1 => $Spacer,
    								 	OBJ_VAL2 => html('Addtional IP Addresses for Anti-Spoofing:'),
    								 	OBJ_VAL3 => $var }] },
    					{	SECTION_END => 1 }
    				);
    			}

			} # end of Office Mode ...

			# Close off Office Mode ...
			push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };

  			
   			#------------------------------------------------
   			# Remote Access - Clientless VPN ...
   			#------------------------------------------------
			if (Version($obj) ge '5.03') {
    			push @{$tobj[$cnt]{$layout}}, (
    				{ SF_BEG => [
    					{	SECTION  => html('Clientless VPN'), 
    						sub      => 1,
    						OBJ_NAME => $name, 
    						STATE    => 0 }
    				] }
    			);
    
    			push @{$tobj[$cnt]{$layout}}, (
    				{ SECTION => [
    					{	OBJ_VAL1 => (GetKey($obj, 'use_clientless_vpn') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html('Support Clientless VPN') }
    				] }
    			);
    			
				if (Version($obj) ge '5.04') {
        			if (GetKey($obj, 'use_clientless_vpn') eq 'true') {
        				push @{$tobj[$cnt]{$layout}}, (
        					{ SECTION => [
        						{	CP3 => 1, C2 => 3, C3 => 2,
        							OBJ_VAL1 => $Spacer,
        							OBJ_VAL2 => html('Certificate for gateway authentication:'),
        							OBJ_VAL3 => html(stripquotes(GetKey($obj, 'use_cert'))) },
        						 {	CP3 => 1, C2 => 2, C3 => 3,
        						 	OBJ_VAL1 => $Spacer,
        							OBJ_VAL2 => html('Client authentication:'),
        							OBJ_VAL3 => $VPNAsk{GetKey($obj, 'clientless_VPN_ask_user_for_certificate')} },
        						 {	CP3 => 1, C2 => 3,
        						 	OBJ_VAL1 => $Spacer,
        							OBJ_VAL2 => html('Number of concurrent servers/processes:'),
        							OBJ_VAL3 => number(GetKey($obj, 'clientless_proc_num')) }
        					] },
        					{ SECTION => [
        						{	OBJ_VAL1 => $Spacer,
        							OBJ_VAL2 => (GetKey($obj, 'accept_3des_for_clientless_vpn') eq 'true')?$CheckTick:$Check,
        							OBJ_VAL3 => html('Accept only 3DES for clientless VPN connections') }
        					] }
        				);
        			}
        		}
    			# Close off Clientless VPN ...
    			push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };
    		}


			#------------------------------------------------
			# Remote Access - SSL Network Extender ...
			#------------------------------------------------
			if (Version($obj) ge '6') {
				push @{$tobj[$cnt]{$layout}}, (
					{ SF_BEG => [
						{	SECTION  => html('SSL Network Extender'), 
							SUB      => 1,
							OBJ_NAME => $name, 
							STATE    => 0 }
					] },
    				{ SECTION => [
    					{	OBJ_VAL1 => (GetKey($obj, 'ssl_ne:ssl_enable') eq 'true')?$CheckTick:$Check,
    						OBJ_VAL2 => html('Support SSL Network Extender') },
    					{	CP3 => 1, INACTIVE => !(GetKey($obj, 'ssl_ne:ssl_enable') eq 'true'),
    						OBJ_VAL2 => html("The gateway authenticates with this certificate:"),
    						OBJ_VAL3 => (GetKey($obj, 'ssl_ne:gw_certificate') eq '')?html('defaultCert'):GetKey($obj, 'ssl_ne:gw_certificate') },
    				] },
    				{ SF_END => 1 }
    			);
    		}
		

			#------------------------------------------------
			# Close off Remote Access ...
			#------------------------------------------------
			push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };

		}


		#------------------------------------------------
		# Authentication ...
		#------------------------------------------------
		if ($FW_Installed) {
			# Firewall installed on object ...
			$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'Authentication');
			$layout  = 'OBJ_'.$layout{'Authentication'};
			if ($section) {
				push @{$tobj[$cnt]{$layout}}, (
					{ 	SF_BEG => 	[{	SECTION  => html('Authentication'), 
										OBJ_NAME => $name, 
										STATE    => ($section eq $_OPEN) }] }
				);
				
				# Authentication Schemes ...
				$val = GetKey($obj, 'host_schemes_val');
				push @{$tobj[$cnt]{$layout}}, (
					{	SECTION_BEGIN => html('Enabled Authentication Schemes:') },
					{	COL_BEG => 1,
						SECTION	=>	[{	OBJ_VAL1 => ($val & $AuthCode{'VPN-1'})?$CheckTick:$Check,
									OBJ_VAL2 => html('VPN-1 & Firewall-1 Password') },
								 {	OBJ_VAL1 => ($val & $AuthCode{'SecurID'})?$CheckTick:$Check,
									OBJ_VAL2 => html('SecurID') },
								 {	OBJ_VAL1 => ($val & $AuthCode{'OS'})?$CheckTick:$Check,
									OBJ_VAL2 => html('OS Password') }] },
					{	COL_NXT => 1,
						SECTION	=>	[{	OBJ_VAL1 => ($val & $AuthCode{'RADIUS'})?$CheckTick:$Check,
									OBJ_VAL2 => html('RADIUS') },
								 {	OBJ_VAL1 => ($val & $AuthCode{'TACACS'})?$CheckTick:$Check,
									OBJ_VAL2 => html('TACACS') }] },
					{ 	COL_END => 1 },
					{	SECTION_END => 1 }
				);
	
				if ((Version($obj) ge '5.0') && ($class !~ /^vs/)) {
    				# Authentication Settings ...
    				push @{$tobj[$cnt]{$layout}}, (
    					{	SECTION_BEGIN => html('Authentication Settings:') },
    					{	SECTION	=>	[{	CP2 => 1,
    									OBJ_VAL1 => html('User Authentication session timeout:'),
    									OBJ_VAL2 => number(GetKey($obj, 'au_timeout')),
    									OBJ_VAL3 => html('Minutes') }] },
    					{	SECTION	=>	[{	OBJ_VAL1 => (GetKey($obj, 'ca_wait_mode') eq 'true')?$CheckTick:$Check,
    									OBJ_VAL2 => html('Enable wait mode for Client Authentication') }] },
    					{	SECTION	=>	[{	CP2 => 1,
    									OBJ_VAL1 => html('Authentication Failure Track:'),
    									OBJ_VAL2 => html($Track{GetKey($obj, 'default_track')}) }] },
    					{	SECTION_END => 1 }
    				);

    				# HTTP Security Server ...
    				push @{$tobj[$cnt]{$layout}}, (
    					{	SECTION_BEGIN => html('HTTP Security Server:') },
    				);
    				if (GetKey($obj, 'http_next_proxy_defined') eq 'true') {
	    				push @{$tobj[$cnt]{$layout}}, (
    						{	SECTION	=> [
    							{	CP3 => 1, CP5 => 1,
    								OBJ_VAL1 => $CheckTick." ".html('Use Next Proxy'),
	    							OBJ_VAL2 => html(' Host:'),
    								OBJ_VAL3 => html(stripquotes(GetKey($obj, 'http_next_proxy_host'))),
    								OBJ_VAL4 => html('Port:'),
    								OBJ_VAL5 => number(GetKey($obj, 'http_next_proxy_port')) }
    						] },
    					);
    				} else {
	    				push @{$tobj[$cnt]{$layout}}, (
    						{	SECTION	=> [
    							{	OBJ_VAL1 => $Check." ".html('Use Next Proxy') }
    						] }
    					);
    				}
    				push @{$tobj[$cnt]{$layout}}, {	SECTION_END => 1 };
    			}
    			
    			# Policy Server ...
				if (GetKey($obj, 'policy_server') eq 'installed') {
					%var = %{obj2html($Policy, GetKey($obj, 'auth_user_groups:Table'), GetKey($obj, 'auth_user_groups:Name'), $_HREF)};
					$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};

    				push @{$tobj[$cnt]{$layout}}, (
    					{ SECTION_BEGIN => html('Policy Server:') },
   						{ SECTION	=> [
    						{	OBJ_VAL1 => html("Users: "),
    							OBJ_VAL2 => $var }
    					] },
    					{ SECTION_END => 1 }
    				);
				}
				
				# End of Authentication 
				push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };
			}
		}
			
		#------------------------------------------------
		# SmartDirectory (LDAP) ...
		#------------------------------------------------
		if ($FW_Installed && (GetKey('properties:firewall_properties:fwldap_UseLDAP') eq 'true')) {
			# Firewall installed on object ...
    		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'SmartDirectory');
    		$layout  = 'OBJ_'.$layout{'SmartDirectory'};
    		if ($section) {
    			push @{$tobj[$cnt]{$layout}}, (
    				{ 	SF_BEG => 	[{	SECTION  => html('SmartDirectory (LDAP)'), 
    									OBJ_NAME => $name, 
    									STATE    => ($section eq $_OPEN) }] }
    			);
    			
    			# Authentication Schemes ...
    			push @{$tobj[$cnt]{$layout}}, (
    				{	SECTION	=>	[{	OBJ_VAL1 => (GetKey($obj, 'display_au_list') eq 'true')?$CheckTick:$Check,
    								OBJ_VAL2 => html('Display list of distinguished names (DNs);for matching UID on login') }] },
    				{	SECTION	=>	[{	CP2 => 1,
    								OBJ_VAL1 => html('Timeout on LDAP requests:'),
    								OBJ_VAL2 => number(GetKey($obj, 'fwldap_RequestTimeout')),
    								OBJ_VAL3 => html('seconds') }] }
    			);
    			
    			# Account Unit Query ...
    			push @{$tobj[$cnt]{$layout}}, (
    				{	SECTION_BEGIN => html('Account Unit Query') },
    				{	SECTION 	=>	[{	OBJ_VAL1 => (GetKey($obj, 'use_custom_au_list') eq 'false')?$MarkDot:$Mark,
    								OBJ_VAL2 => html('All') },
    							 {	OBJ_VAL1 => (GetKey($obj, 'use_custom_au_list') eq 'true')?$MarkDot:$Mark,
    								OBJ_VAL2 => html('Selected Account Units list') }] }
    			);
    			
    			if (GetKey($obj, 'use_custom_au_list') eq 'true') {
    				my @section;
    				my ($ref, $refobj);
    				my ($priority, $priority_index, $default_priority);
    				my ($ldap, $ldapsrv);
    
    				push @section,	{	HEAD => 1, 
    									OBJ_VAL1 => html('Account Unit'),
    									OBJ_VAL2 => html('Host'),
    									OBJ_VAL3 => html('Priority')
    				};
    
    				foreach $ref (GetMembers(GetKey($obj, 'custom_multi_server_au_list'))) {
    					%var  = %{obj2html($Policy, GetKey($ref, 'account_unit:Table'), GetKey($ref, 'account_unit:Name'), $_HREF)};
    					$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    
    					$default_priority = (GetKey($ref, 'default_priority') eq 'true');
    					$priority_index   = GetKey($ref, 'priority_index');
    					$refobj  = GetKey(GetKey($ref, 'account_unit:Table').':'.GetKey($ref, 'account_unit:Name'));
    
    					foreach $ldap (GetMembers(GetKey($refobj, 'ldap_servers'))) {
    						$ldapsrv = GetKey($ldap, 'server:Name');
    						if ($default_priority) {
    							$priority = GetKey($ldap, 'priority');
    						} else {
    							$val = GetKey($ldap, 'other_priorities:'.$priority_index);
    							$priority = GetKey($ldap, 'other_priorities:'.$priority_index);
    						}
    						push @section, {
    							OBJ_VAL1 => $var,
    							OBJ_VAL2 => $ldapsrv,
    							OBJ_VAL3 => number($priority)
    						};
    						$var = '';
    					}
    				}	
    				push @{$tobj[$cnt]{$layout}}, {	C3 => 1, TABLE	=>	\@section }
    			}
    			push @{$tobj[$cnt]{$layout}}, {	SECTION_END => 1 };
    			
    			# End of SmartDirectory (LDAP)
    			push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };
    		}
    	}
	
		#------------------------------------------------
		# SmartView Monitor ...
		#------------------------------------------------
		if (Version($obj) ge '5.03') {
			if (GetKey($obj, 'real_time_monitor') eq 'true') {
				# SmartView monitor product installed ...
				$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'SmartView Monitor');
				$layout  = 'OBJ_'.$layout{'SmartView Monitor'};
				if ($section) {
					if (GetKey($obj, 'real_time_monitor') eq 'true') {
						push @{$tobj[$cnt]{$layout}}, (
							{ 	SF_BEG => 	[{	SECTION  => html('SmartView Monitor'), 
												OBJ_NAME => $name, 
												STATE    => ($section eq $_OPEN) }] }
						);
			
						push @{$tobj[$cnt]{$layout}}, (
							{	TXT	=>	[{	CLASS => 'note_xspc',
											TEXT  => html('Select data to be stored for SmartView Monitor history reports:') }] },
							{	SECTION	=>	[{	OBJ_VAL1 => (GetKey($obj, 'enable_rtm_counters_report') eq 'true')?$CheckTick:$Check,
											OBJ_VAL2 => html('Check Point system counters (e.g. CPU usage, accepted packets)') },
										 {	OBJ_VAL1 => (GetKey($obj, 'enable_rtm_traffic_report_per_connection') eq 'true')?$CheckTick:$Check,
											OBJ_VAL2 => html('Traffic Connections') },
										 {	OBJ_VAL1 => (GetKey($obj, 'enable_rtm_traffic_report') eq 'true')?$CheckTick:$Check,
											OBJ_VAL2 => html('Traffic Throughput (Bytes per second)') }] },
							{	TXT	=>	[{	CLASS => 'note',
											TEXT  => html(';').
													 html('The history database is located on the gateway. This is a cyclic;').
													 html('database that can consume up to 20 MB.') }] },
							{	TXT	=>	[{	CLASS => 'note',
											TEXT  => html(";Note: The implementation of Traffic's history reports may have;").
													 html("performance implications.") }] }
						);
			
						# End of SmartView Monitor
						push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };
					}
				}
			}
		}
	
		#------------------------------------------------
		# UserAuthority Server ...
		#------------------------------------------------
		if ((GetKey($obj, 'UA_gateway') eq 'true') &&
			($FW_Installed || $VPN_Installed)) {
    		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'UserAuthority Server');
    		$layout  = 'OBJ_'.$layout{'UserAuthority Server'};
    		if ($section) {
    			if (GetKey($obj, 'UA_gateway') eq 'true') {
    				push @{$tobj[$cnt]{$layout}}, (
    					{ 	SF_BEG => 	[{	SECTION  => html('UserAuthority Server'), 
    										OBJ_NAME => $name, 
    										STATE    => ($section eq $_OPEN) }] }
    				);
    	
    				# Configuration Method ...
    				push @{$tobj[$cnt]{$layout}}, (
    					{	SECTION_BEGIN => html('Configuration Method') },
    					{	SECTION 	=>	[{	OBJ_VAL1 => (GetKey($obj, 'netso:netso_auto_config') eq 'true')?$MarkDot:$Mark,
    									OBJ_VAL2 => html('Automatic Configuration') },
    								 {	OBJ_VAL1 => (GetKey($obj, 'netso:netso_auto_config') eq 'false')?$MarkDot:$Mark,
    									OBJ_VAL2 => html('Manual Configuration') }] }
    				);
    				
    				if (GetKey($obj, 'netso:netso_auto_config') eq 'false') {
    					push @{$tobj[$cnt]{$layout}}, (
    						{	SECTION	=>	[{	OBJ_VAL1 => $Spacer,
    										OBJ_VAL2 => html('Share identity accross the UserAuthority Server installed on:') }] }
    					);
    	
    					my @section;
    					push @section, 
    						{	OBJ_VAL1 => $Spacer,
    							OBJ_VAL2 => (GetKey($obj, 'netso:netso_vpn_redirect') eq 'true')?$CheckTick:$Check,
    							OBJ_VAL3 => html('VPN Tunnel endpoints when VPN is established') 
    						};
    						
    					$var = '';
    					if (GetKey($obj, 'netso:netso_share_WA_list')) {
    						%var = %{obj2html($Policy, GetKey($obj, 'netso:netso_share_WA_list:Table'), GetKey($obj, 'netso:netso_share_WA_list:Name'), $_HREF)};
    						$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    					}
    					push @section, 
    						{	CP4 => 1, 
    							OBJ_VAL1 => $Spacer,
    							OBJ_VAL2 => (GetKey($obj, 'netso:netso_share_WA_list'))?$CheckTick:$Check,
    							OBJ_VAL3 => html('Servers that share Webaccess Authentication'),
    							OBJ_VAL4 => $var 
    						};
    	
    					$var = '';
    					if (GetKey($obj, 'netso:netso_use_UAS') eq 'true') {
    						%var = %{obj2html($Policy, GetKey($obj, 'netso:netso_UAS:Table'), GetKey($obj, 'netso:netso_UAS:Name'), $_HREF)};
    						$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    					}
    					push @section, 
    						{	CP4 => 1,
    							OBJ_VAL1 => $Spacer,
    							OBJ_VAL2 => (GetKey($obj, 'netso:netso_use_UAS') eq 'true')?$CheckTick:$Check,
    							OBJ_VAL3 => html('Windows Domain Controllers'),
    							OBJ_VAL4 => $var 
    						};
    	
    					$var = '';
    					if (GetKey($obj, 'netso:netso_on_citrix_list')) {
    						%var = %{obj2html($Policy, GetKey($obj, 'netso:netso_on_citrix_list:Table'), GetKey($obj, 'netso:netso_on_citrix_list:Name'), $_HREF)};
    						$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    					}
    					push @section, 
    						{	CP4 => 1,
    							OBJ_VAL1 => $Spacer,
    							OBJ_VAL2 => GetKey($obj, 'netso:netso_on_citrix_list')?$CheckTick:$Check,
    							OBJ_VAL3 => html('Citrix/Microsoft Terminal Servers'),
    							OBJ_VAL4 => $var 
    						};
    	
    					$var = '';
    					if (GetKey($obj, 'netso:netso_other_gw')) {
    						%var = %{obj2html($Policy, GetKey($obj, 'netso:netso_other_gw:Table'), GetKey($obj, 'netso:netso_other_gw:Name'), $_HREF)};
    						$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    					}
    					push @section, 
    						{	CP4 => 1,
    							OBJ_VAL1 => $Spacer,
    							OBJ_VAL2 => GetKey($obj, 'netso:netso_other_gw')?$CheckTick:$Check,
    							OBJ_VAL3 => html('VPN-1/FireWall-1 Gateways'),
    							OBJ_VAL4 => $var
    						};
    	
    					push @{$tobj[$cnt]{$layout}}, {	SECTION	=>	\@section }
    	
    				}
    				push @{$tobj[$cnt]{$layout}}, {	SECTION_END => 1 };
    											
    				# Export Policy ...
    				push @{$tobj[$cnt]{$layout}}, (
    					{	SECTION_BEGIN => html('Export Policy') },
    					{	TXT	=>	[{	CLASS => 'note_xspc',
    									TEXT  => html('Export the following data to externally-managed UserAuthority Servers:') }] },
    					{	COL_BEG => 1 },
    					{	SECTION 	=>	[{	OBJ_VAL1 => (GetKey($obj, 'netso:netso_export_user') eq 'true')?$CheckTick:$Check,
    									OBJ_VAL2 => html('User name') },
    								 {	OBJ_VAL1 => (GetKey($obj, 'netso:netso_export_dn') eq 'true')?$CheckTick:$Check,
    									OBJ_VAL2 => html('DN') }] },
    					{	COL_NXT => 1 },
    					{	SECTION 	=>	[{	OBJ_VAL1 => (GetKey($obj, 'netso:netso_export_group') eq 'true')?$CheckTick:$Check,
    									OBJ_VAL2 => html('User groups') },
    								 {	OBJ_VAL1 => (GetKey($obj, 'netso:netso_export_clientip') eq 'true')?$CheckTick:$Check,
    									OBJ_VAL2 => html('Source IP') }] },
    					{	COL_NXT => 1 },
    					{	SECTION 	=>	[{	OBJ_VAL1 => (GetKey($obj, 'netso:netso_export_scheme') eq 'true')?$CheckTick:$Check,
    									OBJ_VAL2 => html('Authentication Scheme') }] },
    					{	COL_END => 1 },
    					{	SECTION_END => 1 }
    				);
    					
    				# Logging Level ...
    				push @{$tobj[$cnt]{$layout}}, (
    					{	SECTION_BEGIN => html('Logging Level') },
    					{	SECTION	=>	[{	CP2 => 1,
    									OBJ_VAL1 => html('UserAuthority Logging Level:'),
    									OBJ_VAL2 => html($LogLevel{GetKey($obj, 'netso:netso_log_level')}) }] },
    					{	SECTION_END => 1 }
    				);
    				
    				# Close UserAuthority Server
    				push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };
    			}
    		} # End of UserAuthority Server		
		}
		
   		#------------------------------------------------
   		# UserAuthority WebAccess ...
   		#------------------------------------------------
		if (Version($obj) ge '5.02') {
   			if (GetKey($obj, 'UA_WebAccess') eq 'true') {
	       		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'UserAuthority WebAccess');
    			$layout  = 'OBJ_'.$layout{'UserAuthority WebAccess'};
    			if ($section) {
    				push @{$tobj[$cnt]{$layout}}, (
    					{ 	SF_BEG => 	[{	SECTION  => html('UserAuthority WebAccess'), 
    										OBJ_NAME => $name, 
    										STATE    => ($section eq $_OPEN) }] }
    				);
    				
    				%var = %{obj2html($Policy, GetKey($obj, 'web_auth:user_auth_server_ip:Table'), GetKey($obj, 'web_auth:user_auth_server_ip:Name'), $_HREF)};
    				my $server  = (GetKey($obj, 'web_auth:user_auth_server_ip:Name'))?$var{OBJ_ICON}.' '.$var{OBJ_TEXT}:"-";
    				%var = %{obj2html($Policy, GetKey($obj, 'web_auth:user_auth_server_port:Table'), GetKey($obj, 'web_auth:user_auth_server_port:Name'), $_HREF)};
    				my $service = (GetKey($obj, 'web_auth:user_auth_server_port:Name'))?$var{OBJ_ICON}.' '.$var{OBJ_TEXT}:"-";
    				push @{$tobj[$cnt]{$layout}}, (
    					{ SECTION => [
    						{	CP3 => 1,
    							OBJ_VAL1 => html('UserAuthority Server:'),
    							OBJ_VAL2 => ((GetKey($obj, 'web_auth:uas_is_this_machine') eq 'false')?$MarkDot:$Mark).html(' On'),
    							OBJ_VAL3 => $server },
    						{	OBJ_VAL2 => ((GetKey($obj, 'uas_is_this_machine') eq 'true')?$MarkDot:$Mark).html(' On this machine') },
							{	C1 => 2,
								OBJ_VAL1 => html('UserAuthority Server service:'),
    						 	OBJ_VAL2 => $service },
    						{	C1 => 2, CP2 => 1,
    						 	OBJ_VAL1 => html('Action taken for URL outside the scope of the policy:'),
    						 	OBJ_VAL2 => $WebURLAction{GetKey($obj, 'web_auth:action_for_url_external_to_all_policies')} },
    						{	INACTIVE => (GetKey($obj, 'web_auth:action_for_url_external_to_all_policies') ne "redirect"),
    							CP2 => 1, C2 => 2,
    							OBJ_VAL1 => html('Redirect URL to:'),
    						 	OBJ_VAL2 => html(stripquotes(GetKey($obj, 'web_auth:redirect_external_url_to')||'-')) },
    						{	CP3 => 1, C1 => 2,
    						 	OBJ_VAL1 => html('Track:'),
    						 	OBJ_VAL2 => $Track{(GetKey($obj, 'web_auth:track'))} },
    					] }
    				);
    
		    		#------------------------------------------------
		    		# UserAuthority WebAccess - Advanced
		    		#------------------------------------------------
    				push @{$tobj[$cnt]{$layout}}, (
    					{ 	SF_BEG => 	[{	SECTION  => html('Advanced...'), 
    										BUTTON     => 1,
    										OBJ_NAME => $name."WebAccess", 
    										STATE    => 0 }] }
    				);
    				push @{$tobj[$cnt]{$layout}}, (
    					{	SECTION_BEGIN => html('Limits and Timeouts') },
    					{	SECTION 	=>	[{	CP2 => 1,
    									OBJ_VAL1 => html('Maximal client request buffer size:'),
    									OBJ_VAL2 => number(GetKey($obj, 'web_auth:max_client_request_buffer')),
    									OBJ_VAL3 => html('KBytes') },
    								{	CP2 => 1,
    									OBJ_VAL1 => html('Communication to UserAuthority Server timeout:'),
    									OBJ_VAL2 => number(GetKey($obj, 'web_auth:communication_to_uas_timeout')),
    									OBJ_VAL3 => html('milliseconds') },
    								{	CP2 => 1,
    									OBJ_VAL1 => html('HTTP session timeout:'),
    									OBJ_VAL2 => number(GetKey($obj, 'web_auth:http_session_timeout')),
    									OBJ_VAL3 => html('secons') }] },
    					{	SECTION_END => 1 }
    				);
    
    				push @{$tobj[$cnt]{$layout}}, (
    					{	SECTION_BEGIN => html('SSL') },
    					{	SECTION 	=>	[{	CP2 => 1,
    									OBJ_VAL1 => html('SSL redirection methodology:'),
    									OBJ_VAL2 => $SSLredirect{GetKey($obj, 'web_auth:ssl_redirection_methodology')} },
    								{	INACTIVE => (GetKey($obj, 'web_auth:ssl_redirection_methodology') ne 'redirect_to_page'),
    									CP2 => 1,
    									OBJ_VAL1 => html('Redirect to:'),
    									OBJ_VAL2 => (GetKey($obj, 'web_auth:ssl_redirection_methodology') eq 'redirect_to_page')?
    												html(stripquotes(GetKey($obj, 'web_auth:ssl_redirect_page'))):'-' }] },
    					{	SECTION_END => 1 }
    				);
    
    				%var = %{obj2html($Policy, GetKey($obj, 'web_auth:NT_domain_controller:Table'), GetKey($obj, 'web_auth:NT_domain_controller:Name'), $_HREF)};
    				my $server  = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
    				push @{$tobj[$cnt]{$layout}}, (
    					{	SECTION_BEGIN => html('Windows Groups:') },
    					{	SECTION 	=>	[{	OBJ_VAL1 => (GetKey($obj, 'web_auth:get_NT_groups') eq 'true')?$CheckTick:$Check,,
    									OBJ_VAL2 => html('Get Windows group data for Windows users') }] },
    					{	SECTION	=>	[{	INACTIVE => (GetKey($obj, 'web_auth:get_NT_groups') eq 'false'),
    									CP2 => 1,
    									OBJ_VAL1 => html('Windows domain controller:'),
    									OBJ_VAL2 => GetKey($obj, 'web_auth:NT_domain_controller')?$server:"" }] },
    					{	SECTION_END => 1 }
    				);
    
    				push @{$tobj[$cnt]{$layout}}, (
    					{	SECTION_BEGIN => html('Case Sensitivity:') },
    					{	SECTION 	=>	[{	OBJ_VAL1 => (GetKey($obj, 'web_auth:URL_case_sensitive') eq 'true')?$CheckTick:$Check,,
    									OBJ_VAL2 => html('Case sensitivity naming convention for URLs') }] },
    					{	SECTION_END => 1 }
    				);
    									
    				# Close UserAuthority WebAccess - Advanced
    				push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };
    				
    				
    				# Close UserAuthority WebAccess
    				push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };
    				
    			} 
    		} # End of UserAuthority WebAccess
		}

		#------------------------------------------------
		# WebServer ...
		#------------------------------------------------
		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'WebServer');
		$layout  = 'OBJ_'.$layout{'WebServer'};
		if ($section) {
			push @{$tobj[$cnt]{$layout}}, WebServer($Policy, $obj, $name, $type, $section);
		}

		#------------------------------------------------
		# MailServer ...
		#------------------------------------------------
		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'MailServer');
		$layout  = 'OBJ_'.$layout{'WebServer'};
		if ($section) {
			push @{$tobj[$cnt]{$layout}}, MailServer($Policy, $obj, $name, $type, $section);
		}
				
		#------------------------------------------------
		# DNSServer ...
		#------------------------------------------------
		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'DNSServer');
		$layout  = 'OBJ_'.$layout{'DNSServer'};
		if ($section) {
			push @{$tobj[$cnt]{$layout}}, DNSServer($Policy, $obj, $name, $type, $section);
		}
				
		#------------------------------------------------
		# FireWall-1 GX ...
		#------------------------------------------------
		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'FireWall-1 GX');
		$layout  = 'OBJ_'.$layout{'FireWall-1 GX'};
		if ($section) {
			push @{$tobj[$cnt]{$layout}}, Firewall1_GX($Policy, $obj, $name, $type, $section);
		}
	
		#------------------------------------------------
		# Logs and Masters ...
		#------------------------------------------------
		if (Version($obj) ge '5.0') {
			if (($FW_Installed) && (GetKey($obj, 'use_loggers_and_masters') eq 'true')) {
				$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'Logs and Masters');
				$layout  = 'OBJ_'.$layout{'Logs and Masters'};
				if ($section) {
					push @{$tobj[$cnt]{$layout}}, (
						{ 	SF_BEG => 	[{	SECTION  => html('Logs and Masters'), 
											OBJ_NAME => $name, 
											STATE    => ($section eq $_OPEN) }] }
					);
		
					# Local Log Files ...
					my $obj1 = GetKey($obj, 'log_policy');
					my $unit = $DiskSpace{GetKey($obj1, 'alert_free_disk_space_metrics')};
					
					%var = %{obj2html($Policy, GetKey($obj1, 'log_switch_schedule:Table'), GetKey($obj1, 'log_switch_schedule:Name'), $_HREF)};
					$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
					push @{$tobj[$cnt]{$layout}}, (
						{ SECTION_BEGIN => html('Log switch') },
						{ SECTION => [
							{	CP3 => 1,
								OBJ_VAL1 => (GetKey($obj1, 'log_switch_on_file_size') eq 'true')?$CheckTick:$Check,
								OBJ_VAL2 => html('Log switch when file size is:'),
								OBJ_VAL3 => (GetKey($obj1, 'log_switch_on_file_size') eq 'true')?number(GetKey($obj1, 'log_switch_size')):"-",
								OBJ_VAL4 => html('MBytes') },
							{	C3 => 2, CP3 => 1,
								OBJ_VAL1 => (GetKey($obj1, 'scheduled_switch') eq 'true')?$CheckTick:$Check,
								OBJ_VAL2 => html('Schedule log switch to:'),
								OBJ_VAL3 => (GetKey($obj1, 'scheduled_switch') eq 'true')?$var:"" }
						] },
						{ SECTION_END => 1 }
					);
					
					push @{$tobj[$cnt]{$layout}}, (
						{ SECTION_BEGIN => html('Disk space management') },
						{ SECTION => [
							{	CP2 => 1,
								OBJ_VAL1 => html('Measure free disk space in:'),
								OBJ_VAL2 => $unit }
						] },
						{ SECTION => [
							{	CP3 => 1,
								OBJ_VAL1 => (GetKey($obj1, 'log_delete_on_below') eq 'true')?$CheckTick:$Check,
								OBJ_VAL2 => html('Required Free Disk Space:'),
								OBJ_VAL3 => (GetKey($obj1, 'log_delete_on_below') eq 'true')?number(GetKey($obj1, 'log_delete_below_value')):"-",
								OBJ_VAL4 => html($unit) },
							{	INACTIVE => (GetKey($obj1, 'log_delete_on_below') eq 'false'),
								C1 => 4, 
								OBJ_VAL1 => html('Old log files will be deleted until the required free disk space is restored.') },
							{	INACTIVE => (GetKey($obj1, 'log_delete_on_below') eq 'false'),
								CP3 => 1,
								OBJ_VAL2 => ((GetKey($obj1, 'log_keep_on_days') eq 'true')?$CheckTick:$Check).
											html(' Do not delete log files from the last'),
								OBJ_VAL3 => (GetKey($obj1, 'log_keep_on_days') eq 'true')?number(GetKey($obj1, 'log_keep_days_value')):"-",
								OBJ_VAL4 => html('Days') },
							{	CP3 => 1, 
								OBJ_VAL1 => (GetKey($obj1, 'alert_on_disk_space') eq 'true')?$CheckTick:$Check,
								OBJ_VAL2 => html('Alert when free disk space is below:'),
								OBJ_VAL3 => (GetKey($obj1, 'alert_on_disk_space') eq 'true')?number(GetKey($obj1, 'alert_free_disk_space')):"-",
								OBJ_VAL4 => html($unit) },
							{	CP3 => 1, 
								OBJ_VAL1 => (GetKey($obj1, 'stop_logging_on_free_disk_space') eq 'true')?$CheckTick:$Check,
								OBJ_VAL2 => html('Stop logging when free disk space is below:'),
								OBJ_VAL3 => (GetKey($obj1, 'stop_logging_on_free_disk_space') eq 'true')?number(GetKey($obj1, 'min_free_disk_space')):"-",
								OBJ_VAL4 => html($unit) },
							{	INACTIVE => (GetKey($obj1, 'stop_logging_on_free_disk_space') eq 'false'),
								OBJ_VAL2 => ((GetKey($obj1, 'reject_connections') eq 'true')?$CheckTick:$Check).
											html(' Reject all connection when logs are not saved.') }
						] },
						{	SECTION_END => 1 }
					);
		
					# Additional Logging ...
					push @{$tobj[$cnt]{$layout}}, (
						{ 	SF_BEG => 	[{	SECTION  => html('Additional Logging'),
											SUB      => 1,
											OBJ_NAME => $name, 
											STATE    => 0 }] }
					);
		
					%var = %{obj2html($Policy, GetKey($obj1, 'log_forward_target:Table'), GetKey($obj1, 'log_forward_target:Name'), $_HREF)};
					$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
					%var = %{obj2html($Policy, GetKey($obj1, 'log_forward_schedule:Table'), GetKey($obj1, 'log_forward_schedule:Name'), $_HREF)};
					$val = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
					push @{$tobj[$cnt]{$layout}}, (
						{ SECTION_BEGIN => html('Log forwarding settings') },
						{ SECTION => [
							{	CP3 => 1,
								OBJ_VAL1 => (GetKey($obj1, 'forward_logs') eq 'true')?$CheckTick:$Check,
								OBJ_VAL2 => html('Forward log files to Management Server:'),
								OBJ_VAL3 => (GetKey($obj1, 'forward_logs') eq 'true')?$var:'-' },
							{	C3 => 2,
								OBJ_VAL1 => $Spacer,
								OBJ_VAL2 => html('Log forwarding schedule:'),
								OBJ_VAL3 => (GetKey($obj1, 'forward_logs') eq 'true')?$val:'-' }
						] },
						{ SECTION => [
							{	INACTIVE => (GetKey($obj1, 'forward_logs') eq 'false'),
								OBJ_VAL1 => $Spacer,
								OBJ_VAL2 => (GetKey($obj1, 'log_switch_before_forwarding') eq 'true')?$CheckTick:$Check,
								OBJ_VAL3 => html('Perform Log switch before log forwarding') }
						] },
						{	SECTION_END => 1 }
					);
		
					push @{$tobj[$cnt]{$layout}}, (
						{ SECTION_BEGIN => html('Advanced settings') },
						{ SECTION => [
							{	C1 => 2, CP2 => 1,
								OBJ_VAL1 => html('Update Account Log every:'),
								OBJ_VAL2 => number(GetKey($obj1, 'acct_update_interval')),
								OBJ_VAL3 => html('Seconds') },
							{	OBJ_VAL1 => (GetKey($obj1, 'etm_logging') eq 'true')?$CheckTick:$Check,
								OBJ_VAL2 => html('Turn on QoS Logging') },
							{	OBJ_VAL1 => (GetKey($obj1, 'Citrix_ICA_application_detection') eq 'true')?$CheckTick:$Check,
								OBJ_VAL2 => html('Detect new Citrix ICA application names') }
						] },
						{	SECTION_END => 1 }
					);
		
					# Close off Additional Logging ...
					push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };
		
					# Masters ...
					push @{$tobj[$cnt]{$layout}}, (
						{ 	SF_BEG => 	[{	SECTION  => html('Masters'),
											SUB      => 1,
											OBJ_NAME => $name, 
											STATE    => 0 }] }
					);
		
					push @{$tobj[$cnt]{$layout}}, (
						{	SECTION	=>	[{	OBJ_VAL1 => !GetKey($obj, 'masters')?$MarkDot:$Mark,
										OBJ_VAL2 => html('Use local definitions for Masters') },
									 {	OBJ_VAL1 => GetKey($obj, 'masters')?$MarkDot:$Mark,
										OBJ_VAL2 => html('Define Masters') }] }
					);
		
					if (GetKey($obj, 'masters')) {
						my @section;
						foreach $val (GetMembers(GetKey($obj, 'masters'))) {
							%var = %{obj2html($Policy, GetKey($val, 'Table'), GetKey($val, 'Name'), $_HREF)};
							$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
							push @section, { OBJ_VAL1 => $var };
						}
						push @{$tobj[$cnt]{$layout}}, (
							{ 	TXT	=>	[{	CLASS => 'note_xspc',
											TEXT  => html('Use the following Management Stations for fetch and install:') }] },
							{ 	SECTION_BEGIN => chr(0) },
							{	SECTION	=> 	\@section },
							{	SECTION_END => 1 }
						);
					}
					
					# Close off Masters ...
					push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };
		
					# Log Servers ...
					push @{$tobj[$cnt]{$layout}}, (
						{ 	SF_BEG => 	[{	SECTION  => html('Log Servers'),
											SUB      => 1,
											OBJ_NAME => $name, 
											STATE    => 0 }] }
					);
		
					push @{$tobj[$cnt]{$layout}}, (
						{	SECTION	=>	[{	OBJ_VAL1 => (GetKey($obj, 'define_logging_servers') eq 'false')?$MarkDot:$Mark,
										OBJ_VAL2 => html('Use local definitions for Log Servers') },
									 {	OBJ_VAL1 => (GetKey($obj, 'define_logging_servers') eq 'true')?$MarkDot:$Mark,
										OBJ_VAL2 => html('Define Log Servers') },
									 {	OBJ_VAL1 => (GetKey($obj, 'log_servers:self_log_server') eq 'true')?$CheckTick:$Check,
										OBJ_VAL2 => html('Save logs locally, on this machine ('.$name.')') }] }
					);
		
					if (GetKey($obj, 'define_logging_servers') eq 'true') {
						my @section;
						my %logserver;
						my $logserver;
		
						# Find all logservers (log and/or alert) ...
						foreach $val (GetMembers(GetKey($obj, 'log_servers:send_logs_to'))) {
							$logserver{GetKey($val, 'Name')}[0] = $val;
						}
						foreach $val (GetMembers(GetKey($obj, 'log_servers:send_alerts_to'))) {
							$logserver{GetKey($val, 'Name')}[1] = $val;
						}
		
						# Set headers of table with logservers ...
						push @section, {	HEAD => 1,
											OBJ_VAL1 => html('Log Server'),
											OBJ_VAL2 => html('Logs'),
											OBJ_VAL3 => html('Alerts')
						};
						# List all logservers, but eliminate duplicates ...
						foreach $val (keys(%logserver)) {
							$logserver = $logserver{$val}[0] || $logserver{$val}[1];
							%var = %{obj2html($Policy, GetKey($logserver, 'Table'), GetKey($logserver, 'Name'), $_HREF)};
							$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
							push @section, {	OBJ_VAL1 => $var,
												OBJ_VAL2 => ($logserver{$val}[0])?$CheckTick:$Check,
												OBJ_VAL3 => ($logserver{$val}[1])?$CheckTick:$Check
							};
						}
						push @{$tobj[$cnt]{$layout}}, (
							{ 	TXT	=>	[{	CLASS => 'note_xspc',
											TEXT  => html('Always send logs to:') }] },
							{ 	SECTION_BEGIN => chr(0) },
							{	C3 => 1,	TABLE	=> 	\@section },
							{	SECTION_END => 1 }
						);
		
						my @section;
						if (GetKey($obj, 'log_servers:backup_log_servers')) {
							foreach $val (GetMembers(GetKey($obj, 'log_servers:backup_log_servers'))) {
								%var = %{obj2html($Policy, GetKey($val, 'Table'), GetKey($val, 'Name'), $_HREF)};
								$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
								push @section, { OBJ_VAL1 => $var };
							}
						} else {
							push @section, { OBJ_VAL1 => $None }
						}
						push @{$tobj[$cnt]{$layout}}, (
							{ 	TXT	=>	[{	CLASS => 'note_xspc',
											TEXT  => html('When a Log Server is unreachable, send logs to:') }] },
							{ 	SECTION_BEGIN => chr(0) },
							{	SECTION	=> 	\@section },
							{	SECTION_END => 1 }
						);
						undef %logserver;
						undef $logserver;
					}
					
					# Close off Log Servers ...
					push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };
		
					# Clear local variables ...
					undef $obj1;
					undef $unit;
		
					# Close off Logs and Masters ...
					push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };
					
				}	# End of Logs and Masters
			}
		}
		
		#------------------------------------------------
		# Capacity Optimization ...
		#------------------------------------------------
		if ($FW_Installed || $VPN_Installed) {
			# Firewall or VPN-1 is installed ...
			$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'Capacity Optimization');
			$layout  = 'OBJ_'.$layout{'Capacity Optimization'};
			if ($section) {
				push @{$tobj[$cnt]{$layout}}, (
					{ 	SF_BEG => 	[{	SECTION  => html('Capacity Optimization'), 
										OBJ_NAME => $name, 
										STATE    => ($section eq $_OPEN) }] }
				);
	
				# Capacity Optimization ...
				push @{$tobj[$cnt]{$layout}}, (
					{	SECTION_BEGIN => html('Capacity Optimization') },
					{	SECTION	=>	[{	CP2 => 1,
									OBJ_VAL1 => html('Maximum concurrent connections:'),
									OBJ_VAL2 => number(GetKey($obj, 'connections_limit')) }] },
					{	SECTION	=>	[{	OBJ_VAL1 => html('Calculate connections hash table size and memory pool') }] },
					{	SECTION	=>	[{	OBJ_VAL1 => (GetKey($obj, 'calculation_type') eq 'partially_automatic')?$MarkDot:$Mark,
									OBJ_VAL2 => html('Automatically') },
								 {	OBJ_VAL1 => (GetKey($obj, 'calculation_type') eq 'manual')?$MarkDot:$Mark,
									OBJ_VAL2 => html('Manually') }] },
					{	SECTION	=>	[{	C3 => 2, CP3 => 1, INACTIVE => (GetKey($obj, 'calculation_type') ne 'manual'),
									OBJ_VAL1 => $Spacer,
									OBJ_VAL2 => html('Connections hash table size:'),
									OBJ_VAL3 => number(GetKey($obj, 'connections_hashsize')) },
								 {	CP3 => 1, INACTIVE => (GetKey($obj, 'calculation_type') ne 'manual'),
									OBJ_VAL1 => $Spacer,
									OBJ_VAL2 => html('Memory pool size:'),
									OBJ_VAL3 => number(GetKey($obj, 'fw_hmem_size')),
									OBJ_VAL4 => html('MByte') },
								 {	CP3 => 1, INACTIVE => (GetKey($obj, 'calculation_type') ne 'manual'),
									OBJ_VAL1 => $Spacer,
									OBJ_VAL2 => html('Maximum memory pool size:'),
									OBJ_VAL3 => number(GetKey($obj, 'fw_hmem_maxsize')),
									OBJ_VAL4 => html('MByte') }] },
					{	SECTION_END => 1 }
				);
	
				if (Version($obj) ge '5.03') {
    				# VPN Capacity Optimization ...
    				push @{$tobj[$cnt]{$layout}}, (
    					{	SECTION_BEGIN => html('VPN Capacity Optimization') },
    					{	SECTION	=>	[{	CP2 => 1,
    									OBJ_VAL1 => html('Maximum concurrent IKE negotiations:'),
    									OBJ_VAL2 => number(GetKey($obj, 'max_num_negs')) },
    								 {	CP2 => 1,
    									OBJ_VAL1 => html('Maximum concurrent tunnels:'),
    									OBJ_VAL2 => number(GetKey($obj, 'max_concurrent_vpn_tunnels')) }] },
    					{	SECTION_END => 1 }
    				);
    			}
	
				# Close off Capacity Optimization ...
				push @{$tobj[$cnt]{$layout}}, { SF_END => 1 };
			}
		}
		
		#------------------------------------------------
		# Advanced ...
		#------------------------------------------------
		$section = Section($Policy, Object2Page($Policy, 'CheckPoint'), 'CheckPoint', 'Advanced');
		$layout  = 'OBJ_'.$layout{'Advanced'};
		if ($section) {
			push @{$tobj[$cnt]{$layout}}, Advanced($Policy, $obj, $name, $type, $section);
		}

		# Check for empty cells and correct if necessary ...
		if (!defined @{$tobj[$cnt]{OBJ_PRODUCTS}}) { push @{$tobj[$cnt]{OBJ_PRODUCTS}}, { VOID => 1 } }
		if (!defined @{$tobj[$cnt]{OBJ_PROP1}}) { push @{$tobj[$cnt]{OBJ_PROP1}}, { VOID => 1 } }
		if (!defined @{$tobj[$cnt]{OBJ_PROP2}}) { push @{$tobj[$cnt]{OBJ_PROP2}}, { VOID => 1 } }

		#------------------------------------------------
		# COMMENT ...
		#------------------------------------------------
		$tobj[$cnt]{OBJ_COMMENTS} = comment2html(stripquotes(GetKey($obj, 'comments')), $_MAX_COMMENT_LENGTH);

		# Return the number of lines written ...
		# (the object + its members)
		return $members+1;	
		
	} # End of cpobject2html routine

}; # End of checpoint2html routine

1;
