package settings2html;
# ---------------------------------------------
# Package to convert general settings to HTML
# ---------------------------------------------
# version 2.0 for NGX
# ---------------------------------------------

use strict;
use CheckPoint;
use Constants;
use Options;
use object2html;
use Strings;

require Exporter;
our (@ISA, @EXPORT);
@ISA = qw(Exporter);
@EXPORT = qw(HTMLTargets HTMLGlobalProperties);


##########################################################################
# Routine: HTMLTargets
#
# Description:
# Write POLICY INSTALLATION TARGETS to HTML file. 
#
# Parameters:
# $rulebase	Reference to the rulebase in the database structure
# $template	Empty template to be filled for the html page
#
# Returns: 
# $template	Filled template 
##########################################################################
sub HTMLTargets {
	use strict;
	my $rb_name;
	my @targets;
	
	my $rulebase = shift;
	my $template = shift;	# TARGETS.TMPL
	
	# Determine rulebase name ...
	$rb_name = GetKey($rulebase, 'collection:Name');
	
	# Fill header with general variables of rulebase
	$template->param(	RULEBASE     => $rb_name,
						CSSPATH      => Option('CSSPath'),
						MAINLOGO     => Option('MainLogo'),
				);

	if (GetKey('policies_collections', $rb_name.':all_internal_modules') eq 'true') {
		@targets = (txt2html('All internal modules', 'any_object', 'black'));
	} else {
		@targets = @{HTMLMemberList($rb_name, GetKey('policies_collections:'.$rb_name.':installable_targets'), $_HREF)};
	}

	$template->param(TARGETS => \@targets );

	return $template;
}

##########################################################################
# Routine: HTMLGlobalProperties
#
# Description:
# Write GLOBAL PROPERTIES to HTML file. 
#
# Parameters:
# $template	Empty template to be filled for the html page
#
# Returns: 
# $template	Filled template 
##########################################################################
sub HTMLGlobalProperties {
	use strict;
	my ($table, @classes);
	my ($LastModified, $version, $PrimaryMgmt, @SecondaryMgmt);	
	my ($obj, $rb_name, $name);
	my ($var, %var, @var, $val);
	my ($v1, $v2, $v3, $v4, $v5);
	
	my $template = shift;

	# Initialize special marker signs ...
	my $Check      = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/check.png>';
	my $CheckTick  = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/checktick.png>';
	my $Mark       = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/mark.png>';
	my $MarkDot    = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/markdot.png>';
	my $Spacer     = '<IMG STYLE="vertical-align: middle;" SRC='.Option('IconPath').'/spacer.png>';
	my $None       = '<I>(none)</I>';

    # Define Check Point table name to be searched ...	
	$table = 'network_objects';

	# Determine LastModified time/date of global properties ...
	$LastModified = GetKey('properties:firewall_properties:AdminInfo:LastModified:Time');

	# Check all objects in network_objects table to find
	# the primary and secondary management servers ...
	if (ref(GetKey($table))) {
		# table is not empty ...
		foreach $name (keys %{GetKey($table)}) {
			if (GetKey($table, $name, 'primary_management') eq 'true') {
				$PrimaryMgmt = $name;
				$version = Version(GetKey($table, $name));
			} elsif (GetKey($table, $name, 'management') eq 'true') {
				push @SecondaryMgmt, $name
			}
		}
	}

	# Define global variables in template file ...		
	$template->param(	CSSPATH 					=> Option('CSSPath'),
						SCRIPTPATH					=> Option('ScriptPath'),
						MAINLOGO					=> Option('MainLogo'),
						SECTION_GLOBALPROPERTIES	=> 1,
						GLOBALPROPERTIES_REF		=> 'GLOBALPROPERTIES',
						LAST_MODIFIED				=> stripquotes($LastModified),
						PRI_MGMT     				=> $PrimaryMgmt,
						SEC_MGMT					=> join("; ", @SecondaryMgmt)
					);

	$table = GetKey('properties:firewall_properties');
		
	# ----------------------------------
	# CLASS: Firewall-1 ...
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Firewall-1';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='FW';
	my @props;
		push @props, ({ 	
			SECTION_BEGIN => html('Position of the rules in the Rule Base'),
			SECTION => [
				{	CP3 => 1,
				 	OBJ_VAL1 => (GetKey($table, 'fw1enable') eq 'true')?$CheckTick:$Check,
    				OBJ_VAL2 => html('Accept VPN-1 & FireWall-1 control connections:'), 
    				OBJ_VAL3 => $Position{stripquotes(GetKey($table, 'fw1_enable_p'))} },
   				{ 	CP3 => 1, 	
    				OBJ_VAL1 => '',
    				OBJ_VAL2 => ((GetKey($table, 'raccessenable') eq 'true')?$CheckTick:$Check).
    							html(' Accept Remote Access control connections:'), 
    				OBJ_VAL3 => $Position{stripquotes(GetKey($table, 'r_access_enable_p'))} },
   				{ 	INACTIVE => ($version lt '6'),
   				 	CP3 => 1, 	
    				OBJ_VAL1 => '',
    				OBJ_VAL2 => ((GetKey($table, 'cpridenable') eq 'true')?$CheckTick:$Check).
    							html(' Accept SmartUpdate connections:'), 
    				OBJ_VAL3 => $Position{stripquotes(GetKey($table, 'cpridenable_p'))} },
   				{	CP3 => 1,
    				OBJ_VAL1 => (GetKey($table, 'outgoing') eq 'true')?$CheckTick:$Check,
    				OBJ_VAL2 => html('Accept outgoing packets originating from Gateway:'), 
    				OBJ_VAL3 => $Position{stripquotes(GetKey($table, 'outgoing_p'))} },
    			{	CP3 => 1, 	
    				OBJ_VAL1 => (GetKey($table, 'rip') eq 'true')?$CheckTick:$Check,
    				OBJ_VAL2 => html('Accept RIP:'), 
    				OBJ_VAL3 => $Position{stripquotes(GetKey($table, 'rip_p'))} },
   				{ 	CP3 => 1, 	
    				OBJ_VAL1 => (GetKey($table, 'domain_udp') eq 'true')?$CheckTick:$Check,
    				OBJ_VAL2 => html('Accept Domain Name over UDP (Queries):'),
    				OBJ_VAL3 => $Position{stripquotes(GetKey($table, 'domain_udp_p'))} },
    			{	CP3 => 1, 	
    				OBJ_VAL1 => (GetKey($table, 'rip') eq 'true')?$CheckTick:$Check,
    				OBJ_VAL2 => html('Accept Domain Name over TCP (Zone Transfer):'), 
    				OBJ_VAL3 => $Position{stripquotes(GetKey($table, 'rip_p'))} },
    			{	CP3 => 1,
    				OBJ_VAL1 => (GetKey($table, 'icmpenable') eq 'true')?$CheckTick:$Check,
    				OBJ_VAL2 => html('Accept ICMP requests:'), 
    				OBJ_VAL3 => $Position{stripquotes(GetKey($table, 'icmpenable_p'))} },
    			{	CP3 => 1, 	
    				OBJ_VAL1 => (GetKey($table, 'dagdhcpenable') eq 'true')?$CheckTick:$Check,
    				OBJ_VAL2 => html("Accept dynamic address Modules'DHCP traffic:"), 
    				OBJ_VAL3 => $Position{stripquotes(GetKey($table, 'dagdhcpenable_p'))} },
    			{	CP3 => 1, 	
    				OBJ_VAL1 => (GetKey($table, 'vrrp') eq 'true')?$CheckTick:$Check,
    				OBJ_VAL2 => html("Accept VRRP packets originating from cluster members:").html(";(VSX Nokia VRRP)"), 
    				OBJ_VAL3 => $Position{stripquotes(GetKey($table, 'vrrp_p'))} }
    		],
			SECTION_END => 1 }
        );
        
		push @props, ({ 
			SECTION_BEGIN => html('Track'),
			SECTION => [
				{	CP1 => 1,
				 	OBJ_VAL1 => (GetKey($table, 'log_implied_rules') eq 'true')?$CheckTick:$Check,
				 	OBJ_VAL2 => html("Log Implied Rules") }
			],
			SECTION_END => 1 }
		);
	
	$class{GP_PROPS}=\@props;

	push @classes, \%class;
	
	# ----------------------------------
	# CLASS: Firewall-1 - Security Server ...
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Firewall-1';
	$class{GP_SUBCLASS}='Security Server';
	$class{GP_CLASS}='FW-SS';

	my @props;
		push @props, 
			{ SECTION => [
				{ 	CP2 => 1,
					OBJ_VAL1 => html('Telnet welcome message file: '), 
        			OBJ_VAL2 => stripquotes(GetKey($table, 'telnet_msg')) },
				{ 	CP2 => 1,
    				OBJ_VAL1 => html('FTP welcome message file: '), 
    				OBJ_VAL2 => stripquotes(GetKey($table, 'ftp_msg')) },
				{ 	CP2 => 1,
    				OBJ_VAL1 => html('Rlogin welcome message file: '), 
    				OBJ_VAL2 => stripquotes(GetKey($table, 'rlogin_msg')) },
				{ 	CP2 => 1,
    				OBJ_VAL1 => html('Client authentication welcome file: '), 
    				OBJ_VAL2 => stripquotes(GetKey($table, 'clnt_auth_msg')) },
				{ 	CP2 => 1,
    				OBJ_VAL1 => html('SMTP welcome message: '), 
    				OBJ_VAL2 => stripquotes(GetKey($table, 'smtp_msg')) },
				{ 	INACTIVE => ($version lt '6'),
				 	CP2 => 1,
    				OBJ_VAL1 => html('MDQ welcome message: '), 
    				OBJ_VAL2 => stripquotes(GetKey($table, 'mdq_msg')) }
    		] };


		push @props, (
			{ 	SECTION_BEGIN => html('HTTP:') },
			{	SECTION => 	[
					{	OBJ_VAL1 => html('Note:'),
						OBJ_VAL2 => html('These settings apply only to firewalled gateways prior to NG.;').
									html('For later versions, these settings can be configured in the Node;').
									html('Properties.') }
				] },
			{	SECTION => 	[
					{	CP3 => 1, CP5 => 1,
						OBJ_VAL1 => html('HTTP next proxy:        '),
						OBJ_VAL2 => html('Host:'),
						OBJ_VAL3 => GetKey($table, 'http_next_proxy_host'),
						OBJ_VAL4 => html('Port:'),
						OBJ_VAL5 => GetKey($table, 'http_next_proxy_port') }
				] },
			{ 	SECTION_END => 1 }
							
		);
							
		push @props, { SECTION_BEGIN => html('HTTP Server') };
		my @section;
        	push @section, { 	HEAD => 1,
        						OBJ_VAL1 => html('Logical Name'), 
        						OBJ_VAL2 => html('Host'),
        						OBJ_VAL3 => html('Port'),
        						OBJ_VAL4 => html('Reauthentication')};

			foreach $var (keys %{GetKey($table, 'http_servers')}) {
				if (GetKey($table, 'http_servers:'.$var.':AdminInfo:ClassName') eq 'http_server') {
    				$obj=GetKey($table, 'http_servers:'.$var);
    	        	push @section, {	OBJ_VAL1 => html($var), 
        	    						OBJ_VAL2 => html(GetKey($obj, 'host')),
            							OBJ_VAL3 => html(GetKey($obj, 'port')),
            							OBJ_VAL4 => html(GetKey($obj, 'reauth_requests'))};
				}
			}
		push @props, { TABLE => \@section, C4 => 1 };
		push @props, { SECTION_END => 1 };
			
	
	$class{GP_PROPS}=\@props;
		
	push @classes, \%class;
	
	# ----------------------------------
	# CLASS: Firewall-1 - VoIP ...
	# ----------------------------------

	# This entire section was removed in version 6 (NGX) ...
	if ($version lt '6') {
    	
    	my %class;
    	$class{GP_MAINCLASS}='Firewall-1';
    	$class{GP_SUBCLASS}='VoIP';
    	$class{GP_CLASS}='FW-VI';
    
    	my @props;
    		push @props, 
    			{ SECTION => [
    				{	OBJ_VAL1 => (GetKey($table, 'h323_log_conn') eq 'true')?$CheckTick:$Check,
    				 	OBJ_VAL2 => html('Log VoIP connection') }
    			] }; 
    
    		push @props, (
    			{ SECTION_BEGIN => html('SIP') },
        		{ SECTION => [
        			{	OBJ_VAL1 => (GetKey($table, 'sip_allow_redirect') eq 'true')?$CheckTick:$Check,
        				OBJ_VAL2 => html('Allow calls using a proxy or a redirect server') },
        			{	OBJ_VAL1 => (GetKey($table, 'sip_allow_instant_messages') eq 'true')?$CheckTick:$Check,
        				OBJ_VAL2 => html('Allow SIP-based Instant Messaging') },
        			{	OBJ_VAL1 => (GetKey($table, 'sip_enforce_security_reinvite') eq 'true')?$CheckTick:$Check,
        				OBJ_VAL2 => html('Allow the destination to re-invite calls') }
        		] },
        		{ SECTION => [
        			{	CP2 => 1,
        				OBJ_VAL1 => html('Maximum invitations per call (from both directions):'),
        				OBJ_VAL2 => GetKey($table, 'sip_max_reinvite') }
        		] },
        		{ SECTION_END => 1 }
        	);
    
    		push @props, (
    			{ SECTION_BEGIN => html('H.323') },
        		{ SECTION => [
        			{	OBJ_VAL1 => (GetKey($table, 'fwh323_allow_redirect') eq 'true')?$CheckTick:$Check,
        				OBJ_VAL2 => html('Allow to re-direct connections') },
        			{	OBJ_VAL1 => (GetKey($table, 'sip_allow_instant_messages') eq 'true')?$CheckTick:$Check,
        				OBJ_VAL2 => html('Prevent blank source phone numbers for gatekeeper connections') },
        			{	OBJ_VAL1 => (GetKey($table, 'allow_h323_t120') eq 'true')?$CheckTick:$Check,
        				OBJ_VAL2 => html('Enable dynamic T.120') }
        		] },
        		{ SECTION_END => 1 }
        	);
    
    	$class{GP_PROPS}=\@props;
    		
    	push @classes, \%class;
	}

	# ----------------------------------
	# CLASS: Network Address Translation ...
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='NAT - Network Address Translation';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='NAT';

	my @props;
		push @props, (
			{ SECTION_BEGIN => html('Automatic NAT rules') },
    		{ SECTION => [
    			{	OBJ_VAL1 => (GetKey($table, 'nat_automatic_rules_merge') eq 'true')?$CheckTick:$Check,
    				OBJ_VAL2 => html('Allow bi-directional NAT') },
    			{	OBJ_VAL1 => (GetKey($table, 'nat_dst_client_side') eq 'true')?$CheckTick:$Check,
    				OBJ_VAL2 => html('Translate destination on client side') },
    			{	OBJ_VAL1 => (GetKey($table, 'nat_automatic_arp') eq 'true')?$CheckTick:$Check,
    				OBJ_VAL2 => html('Automatic ARP configuration') },
       			{ 	INACTIVE => ($version lt '6'),
        			OBJ_VAL1 => '',
        			OBJ_VAL2 => ((GetKey($table, 'arp_proxy_merge') eq 'true')?$CheckTick:$Check).
        						html(' Merge manual proxy ARP configuration') }
        	] },
        	{ SECTION_END => 1 }
    	);

		push @props, (
			{ SECTION_BEGIN => html('Manual NAT rules') },
			{ SECTION => [
				{ 	OBJ_VAL1 => (GetKey($table, 'nat_dst_client_side_manual') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Translate destination on client side') }
			] },
        	{ SECTION_END => 1 }
		);

		$v1 = stripquotes(GetKey($table, 'ip_pool_log'));
		$v2 = stripquotes(GetKey($table, 'log_ip_pool_allocation'));
		push @props, (
			{ SECTION_BEGIN => html('IP Pool NAT') },
			{ SECTION => [
				{ 	C2 => 4,
					OBJ_VAL1 => (GetKey($table, 'enable_ip_pool') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Enable IP Pool NAT for SecuRemote/SecureClient and gateway to gateway connections') }
			] },
			{ SECTION => [
				{ 	OBJ_VAL1 => '',
				 	OBJ_VAL2 => html('Address exhaustion track:'),
					OBJ_VAL3 => (($v1 eq 'none')?$MarkDot:$Mark).html(' None'), 
					OBJ_VAL4 => (($v1 eq 'IP Exhaustion Log')?$MarkDot:$Mark).html(' Log'), 
					OBJ_VAL5 => (($v1 eq 'IP Exhaustion Alert')?$MarkDot:$Mark).html(' Alert') },
				{	OBJ_VAL1 => '',
				 	OBJ_VAL2 => html('Address allocation and release track:'),
					OBJ_VAL3 => (($v2 eq 'none')?$MarkDot:$Mark).html(' None'), 
					OBJ_VAL4 => (($v2 eq 'IP Allocation Log')?$MarkDot:$Mark).html(' Log') }
			] },
			{ SECTION_END => 1 }
		);

	$class{GP_PROPS}=\@props;
		
	push @classes, \%class;
	
	# ----------------------------------
	# CLASS: Authentication
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Authentication';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='AUTH';

	my @props;
	push @props, (
		{ SECTION_BEGIN => html('Failed authentication attempts') },
    	{ SECTION => [
    		{	CP2 => 1,
    		 	OBJ_VAL1 => html('Terminate rlogin connection after'),
    			OBJ_VAL2 => html(GetKey($table, 'rlogin_max_auth_allowed')),
    			OBJ_VAL3 => html('attempts') },
    		{	CP2 => 1,
    		  	OBJ_VAL1 => html('Terminate telnet connection after'),
    			OBJ_VAL2 => html(GetKey($table, 'telnet_max_auth_allowed')),
    			OBJ_VAL3 => html('attempts') }
    	] },
		{ SECTION => [
			{	CP2 => 1,
			 	OBJ_VAL1 => html('Terminate Client Authentication connection after'),
    			OBJ_VAL2 => html(GetKey($table, 'client_max_auth_allowed')),
    			OBJ_VAL3 => html('attempts') },
    		{	CP2 => 1,
			 	OBJ_VAL1 => html('Terminate Session Authentication connection after'),
    			OBJ_VAL2 => html(GetKey($table, 'session_max_auth_allowed')),
    			OBJ_VAL3 => html('attempts') }
    	] },
		{ SECTION_END => 1 }
    );

	push @props, (
		{ SECTION_BEGIN => html('Authentication of users with certificates') },
    	{ SECTION => [
    		{	OBJ_VAL1 => (GetKey($table, 'enforce_suffix_for_internal_users') eq 'true')?$CheckTick:$Check,
				OBJ_VAL2 => html('Authenticate internal users with this suffix only:') }
		] },
    	{ SECTION => [
    		{	CP2 => 1,
			 	OBJ_VAL1 => $Spacer,
				OBJ_VAL2 => html(stripquotes(GetKey($table, 'allowed_suffix_for_internal_users'))) }
		] },
    	{ SECTION => [
    		{	CP2 => 1,
    			OBJ_VAL1 => html('Users certificates which were initiated but not pulled, will expire after'),
    			OBJ_VAL2 => html(GetKey($table, 'auth_validity_days')),
    			OBJ_VAL3 => html('days') }
    	] },
		{ SECTION_END => 1 }
    );

	# The next two sections were added in version 6.0 (NGX) ...
	if ($version ge '6') {
		push @props, (
			{ SECTION_BEGIN => html('Brute Force password guessing protection') },
         	{ SECTION => [
         		{	OBJ_VAL1 => (GetKey($table, 'authentication_defense_enable') eq 'true')?$CheckTick:$Check,
					OBJ_VAL2 => html('Enable Delayed Authentication') }
			] },
        	{ SECTION => [
        		{	CP2 => 1,
        			OBJ_VAL1 => html('Delay each authentication attempt by'),
        			OBJ_VAL2 => html(GetKey($table, 'authentication_defense_delay')),
        			OBJ_VAL3 => html('msec') }
        	] },
			{ SECTION_END => 1 }
		);

		$v1 = GetKey($table, 'default_track:Name');
		push @props, (
			{ SECTION_BEGIN => html('Early Versions Compatibility') },
			{ SECTION => [
				{	OBJ_VAL1 => html('Note:'),
					OBJ_VAL2 => html('These settings apply only to firewalled gateways prior to NG.;').
								html('For later versions, these settings can be configured in the Node;').
								html('Properties.') }
			] },
        	{ SECTION => [
        		{	CP2 => 1,
        			OBJ_VAL1 => html('User Authentication session timeout'),
        			OBJ_VAL2 => html(GetKey($table, 'au_timeout')),
        			OBJ_VAL3 => html('minutes') }
        	] },
        	{ SECTION => [
        		{	OBJ_VAL1 => (GetKey($table, 'ca_wait_mode') eq 'true')?$CheckTick:$Check,
					OBJ_VAL2 => html('Enable wait mode for Client Authentication') }
			] },
			{ SECTION => [
				{ 	OBJ_VAL1 => '',
				 	OBJ_VAL2 => html('Authentication Failure Track:'),
					OBJ_VAL3 => (($v1 eq 'None')?$MarkDot:$Mark).html(' None'), 
					OBJ_VAL4 => (($v1 eq 'Auth')?$MarkDot:$Mark).html(' Log'), 
					OBJ_VAL5 => (($v1 eq 'AuthAlert')?$MarkDot:$Mark).html(' Alert') }
			] },
			{ SECTION_END => 1 }
		);
	}

	$class{GP_PROPS}=\@props;
		
	push @classes, \%class;
	
	# ----------------------------------
	# CLASS: VPN - Early Versions Compatibility
	# ----------------------------------
	if ($version ge '6') {
    	my %class;
    	$class{GP_MAINCLASS}='VPN';
    	$class{GP_SUBCLASS}='Early Versions Compatibility';
    	$class{GP_CLASS}='VPN-EARLY';
    	
    	my @props;
    		push @props, (
    			{ SECTION_BEGIN => html('IKE') },
				{ SECTION => [
        			{	CP2 => 1,
            			OBJ_VAL1 => html('Renegotiate IKE (phase 1) security associations every'),
            			OBJ_VAL2 => html(GetKey($table, 'isakmpphase1reneg')),
            			OBJ_VAL3 => html('minutes') },
           			{ 	CP2 => 1,
           			  	OBJ_VAL1 => html('Renegotiate IPsec (IKE phase 2) security associations every'),
           				OBJ_VAL2 => html(GetKey($table, 'isakmpphase2reneg')),
           				OBJ_VAL3 => html('seconds') },
           		] },
            	{ SECTION => [
       				{	OBJ_VAL1 => html('Note: These settings apply only to VPN modules prior to NG.') }
           		] },
       			{ SECTION_END => 1 }
           	); 
    
    		push @props, (
    			{ SECTION_BEGIN => html('Resolving mechnanism') },
    			{ SECTION => [
    				{	OBJ_VAL1 => (GetKey($table, 'resolve_interface_ranges_GW') eq 'true')?$MarkDot:$Mark,
        				OBJ_VAL2 => html('Static calculation based on network topology') },
            		{ 	OBJ_VAL1 => (GetKey($table, 'resolve_interface_ranges_GW') eq 'false')?$MarkDot:$Mark,
    				 	OBJ_VAL2 => html('Dynamic interface resolving mechanism') }
            	] },
            	{ SECTION => [
           			{	OBJ_VAL1 => html('Note: These settings apply only to VPN modules of version;').
           							html('NG with Application Intelligence R55/R55W.') }
            	] },
    			{ SECTION_END => 1 }
           	); 
    
    	$class{GP_PROPS}=\@props;
    		
    	push @classes, \%class;
    }
	
	# ----------------------------------
	# CLASS: VPN - Advanced
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='VPN - Advanced';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='VPN-ADV';
	
	my @props;
	push @props, (
		{ SECTION => [
			{	OBJ_VAL1 => (GetKey($table, 'enable_mep_configuration') eq 'true')?$CheckTick:$Check,
		 		OBJ_VAL2 => html('Enable Backup Gateway') },
			 {	OBJ_VAL1 => (GetKey($table, 'vpn_peer_ls_GW') eq 'true')?$CheckTick:$Check,
		    	OBJ_VAL2 => html('Enable load distribution for Multiple Entry Points configurations (Site to Site connections)') },
			 {	OBJ_VAL1 => (GetKey($table, 'acceptdecrypt') eq 'true')?$CheckTick:$Check,
			 	OBJ_VAL2 => html('Enable decrypt on accept for gateway to gateway traffic') }
		] }
	); 

	push @props, (
		{ SECTION_BEGIN => html('CRL Grace Period') },
		{ SECTION => [
			{	CP2 => 1,
    			OBJ_VAL1 => html('Grace period before the CRL is valid:'),
    			OBJ_VAL2 => html(GetKey($table, 'crl_start_grace')),
    			OBJ_VAL3 => html('seconds') },
   			 { 	CP2 => 1,
   			  	OBJ_VAL1 => html('Grace period after the CRL is no longer valid:'),
   				OBJ_VAL2 => html(GetKey($table, 'crl_end_grace')),
   				OBJ_VAL3 => html('seconds') },
   			 { 	CP2 => 1,
			 	OBJ_VAL1 => html('Grace period extension for SecuRemote/SecureClient:'),
   				OBJ_VAL2 => html(GetKey($table, 'sr_grace_period')),
   				OBJ_VAL3 => html('seconds') }
   		] },
		{ SECTION_END => 1 }
    );

	# This section was removed in version 6 (NGX) ...
	if ($version lt '6') {
		push @props, (
			{ SECTION_BEGIN => html('Resolving mechanism') },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($table, 'resolve_interface_ranges_GW') eq 'true')?$MarkDot:$Mark,
					OBJ_VAL2 => html("Enable VPN-1 gateways to calculate statically peer gateway's best interface") },
				{ 	OBJ_VAL1 => (GetKey($table, 'resolve_interface_ranges_GW') eq 'true')?$Mark:$MarkDot,
					OBJ_VAL2 => html("Enable dynamic interface resolving per gateway") }
			] },
			{ SECTION_END => 1 }
		);
	}
	
	push @props, (
		{ SECTION_BEGIN => html('IKE Denial of Service protection') },
		{ SECTION => [
			{	CP2 => 1,
			 	OBJ_VAL1 => html('Support IKE DoS protection from identified source'),
    			OBJ_VAL2 => html($IKEDoS{GetKey($table, 'ike_dos_protection_identified_initiator')}) },
    		{ 	CP2 => 1,
			 	OBJ_VAL1 => html('Support IKE DoS protection from unidentified source'),
    			OBJ_VAL2 => html($IKEDoS{GetKey($table, 'ike_dos_protection_unidentified_initiator')}) }
    	] },
    	{ SECTION_END => 1 }
    );

	if ($version ge '6') {
    	push @props, (
			{ SECTION_BEGIN => html('Link Selection settings') },
    		{ SECTION => [
    			{ 	CP2 => 1,
    			 	OBJ_VAL1 => html('Domain name for DNS resolving:'),
        			OBJ_VAL2 => html(GetKey($table, 'dns_resolving_domain_name')) }
        	] },
			{ SECTION_END => 1 }
    	);
	}

	$class{GP_PROPS}=\@props;
		
	push @classes, \%class;
	
	# ----------------------------------
	# CLASS: VPN-1 Net
	# ----------------------------------
	if ($version lt '6') {
    	my %class;
    	$class{GP_MAINCLASS}='VPN-1 Net';
    	$class{GP_SUBCLASS}='';
    	$class{GP_CLASS}='VPN-1-Net';
    	
    	my @props;
    		push @props, { TXT => [{	CLASS => 'note_xspc',
    									TEXT => html('VPN-1 Net Gateways Configuration') }] };
    
    		push @props, (
    			{ SECTION_HEADER => html('Security Policy') },
    			{ SECTION => [{	OBJ_VAL1 => (GetKey($table, 'vpnddcate_policy') eq 'allow_all')?$MarkDot:$Mark,
        					OBJ_VAL2 => html('Accept all connections') },
            			 { 	OBJ_VAL1 => (GetKey($table, 'vpnddcate_policy') eq 'outbound_and_encrypted')?$MarkDot:$Mark,
    					 	OBJ_VAL2 => html('Accept all encrypted and all outbound connections') },
            			 { 	OBJ_VAL1 => (GetKey($table, 'vpnddcate_policy') eq 'encrypted_only')?$MarkDot:$Mark,
    						OBJ_VAL2 => html('Accept encrypted connections') }, 
            			 { 	OBJ_VAL1 => (GetKey($table, 'vpnddcate_policy') eq 'block_all')?$MarkDot:$Mark,
            			 	OBJ_VAL2 => html('Block all connections') }] }
           	); 
    
    		push @props, (
    			{ SECTION_HEADER => html('NAT') },
    			{ SECTION => [{	OBJ_VAL1 => (GetKey($table, 'vpnddcate_nat') eq 'none')?$MarkDot:$Mark,
    						OBJ_VAL2 => html('Do not hide any connections (no NAT)') }, 
            			 { 	OBJ_VAL1 => (GetKey($table, 'vpnddcate_nat') eq 'hide_clear')?$MarkDot:$Mark,
    					 	OBJ_VAL2 => html('Hide only Non-Encrypted connections') }, 
            			 { 	OBJ_VAL1 => (GetKey($table, 'vpnddcate_nat') eq 'hide_all')?$MarkDot:$Mark,
    					 	OBJ_VAL2 => html('Hide all connections') }] }
    		); 
    
    		push @props, (
    			{ SECTION_HEADER => html('Tracking') },
    			{ SECTION => [{	CP2 => 1,
    					 	OBJ_VAL1 => html('Accepted Traffic'),
            				OBJ_VAL2 => html($Track{GetKey($table, 'vpnddcate_track_accept')}) },
            			 { 	CP2 => 1,
    					 	OBJ_VAL1 => html('Dropped Traffic'),
            				OBJ_VAL2 => html($Track{GetKey($table, 'vpnddcate_track_drop')}) }] }
            );
    
    		push @props, (
    			{ SECTION_HEADER => html('Remote Device Configuration') },
    			{ SECTION => [{	OBJ_VAL1 => (GetKey($table, 'enable_ssl_conn_to_vpn1_net') eq 'true')?$CheckTick:$Check,
        					OBJ_VAL2 => html('Enable HTTPS connections to VPN-1 Net Modules') }, 
        		 		 { 	OBJ_VAL1 => (GetKey($table, 'enable_ssh_conn_to_vpn1_net') eq 'true')?$CheckTick:$Check,
        				 	OBJ_VAL2 => html('Enable SSH connections to VPN-1 Net Modules') }] }
        	); 
    
    	$class{GP_PROPS}=\@props;
    		
    	push @classes, \%class;
    }
	
	# ----------------------------------
	# CLASS: VPN-1 Edge/Embedded Gateway
	# ----------------------------------
	if ($version ge '6') {
    	my %class;
    	$class{GP_MAINCLASS}='VPN-1 Edge/Embedded Gateway';
    	$class{GP_SUBCLASS}='';
    	$class{GP_CLASS}='VPN-1-Edge';
    	
    	my @props;
    		push @props, (
    			{ SECTION_BEGIN => html('General Configuration') },
    			{ SECTION => [
    				{	CP2 => 1,
    					OBJ_VAL1 => html('Update configuration settings every'),
    					OBJ_VAL2 => (GetKey($table, 'sofaware_update_config_interval')/60),
    					OBJ_VAL3 => html('minutes') },
    				{	CP2 => 1,
    					OBJ_VAL1 => ((GetKey($table, 'sofaware_use_log_interval') eq 'true')?$CheckTick:$Check).
    								html(' Limit log sending to once every'),
    					OBJ_VAL2 => (GetKey($table, 'sofaware_log_send_interval')),
    					OBJ_VAL3 => html('seconds') }
    			] },
    			{ SECTION_END => 1 }
    		);

			# Find Server objects, if available ...
			$v1 = ""; $v2 = ""; $v3 = "";
			if (GetKey($table, 'sofaware_ufp_properties:sofaware_enable_ufp_properties') eq 'true') {
				%var = %{obj2html('_GLOBAL', GetKey($table, 'sofaware_ufp_properties:sofaware_ufp_server:Table'), GetKey($table, 'sofaware_ufp_properties:sofaware_ufp_server:Name'), $_HREF)};
				$v1  = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
			}
			if (GetKey($table, 'sofaware_cvp_properties:sofaware_enable_cvp_properties') eq 'true') {
				%var = %{obj2html('_GLOBAL', GetKey($table, 'sofaware_cvp_properties:sofaware_cvp_server:Table'), GetKey($table, 'sofaware_cvp_properties:sofaware_cvp_server:Name'), $_HREF)};
				$v2  = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
				
				if (GetKey($table, 'sofaware_cvp_properties:sofaware_support_smtp') eq 'true') {
					%var = %{obj2html('_GLOBAL', GetKey($table, 'sofaware_cvp_properties:sofaware_smtp_server:Table'), GetKey($table, 'sofaware_cvp_properties:sofaware_smtp_server:Name'), $_HREF)};
					$v3  = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
				}
			}
    		push @props, (
    			{ SECTION_BEGIN => html('Content Filtering') },
    			{ SECTION => [
    				{	OBJ_VAL1 => (GetKey($table, 'sofaware_ufp_properties:sofaware_enable_ufp_properties') eq 'true')?$CheckTick:$Check,
    					OBJ_VAL2 => html('Centralized web filtering') },
    				{	INACTIVE => ($v1 eq ""),
    					CP3 => 1,
    					OBJ_VAL1 => $Spacer,
    					OBJ_VAL2 => html('OPSEC UFP server'),
    					OBJ_VAL3 => $v1 },
    				{	OBJ_VAL1 => (GetKey($table, 'sofaware_cvp_properties:sofaware_enable_cvp_properties') eq 'true')?$CheckTick:$Check,
    					OBJ_VAL2 => html('Centralized E-mail Antivirus') },
    				{	INACTIVE => ($v2 eq ""),
    					CP3 => 1,
    					OBJ_VAL1 => $Spacer,
    					OBJ_VAL2 => html('OPSEC CVP server'),
    					OBJ_VAL3 => $v2 },
    				{	OBJ_VAL1 => $Spacer,
    					OBJ_VAL2 => ((GetKey($table, 'sofaware_cvp_properties:sofaware_support_pop3') eq 'true')?$CheckTick:$Check).
    								html(' Support POP3') },
    				{	OBJ_VAL1 => $Spacer,
    					OBJ_VAL2 => ((GetKey($table, 'sofaware_cvp_properties:sofaware_support_smtp') eq 'true')?$CheckTick:$Check).
    								html(' Support SMTP') },
    				{	INACTIVE => ($v3 eq ""),
    					OBJ_VAL1 => $Spacer,
    					OBJ_VAL2 => $Spacer.html(' SMTP server'),
    					OBJ_VAL3 => $v3 },
    			] },
    			{ SECTION => [
    				{	OBJ_VAL1 => html('Note:'),
    					OBJ_VAL2 => html('Per edge configuration will be done via the Content Filtering tab of the;').
    								html('Edge Gateway object.') }
    			] },
    			{ SECTION_END => 1 }
    		);

    		push @props, (
    			{ SECTION_BEGIN => html('Access control') },
    			{ SECTION => [
    				{	OBJ_VAL1 => (GetKey($table, 'sofaware_unrestricted_behavior') eq 'true')?$CheckTick:$Check,
    					OBJ_VAL2 => html('Apply unrestricted mode (allow all encrypted traffic and disable NAT);').
    								html('to VPN-1 Edge/Embedded Gateways.') }
    			] },
    			{ SECTION_END => 1 }
    		);
    
    	$class{GP_PROPS}=\@props;
    		
    	push @classes, \%class;
    }
	
	# ----------------------------------
	# CLASS: Remote Access
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Remote Access - VPN-1 SecuRemote/SecureClient';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='RA-SR';
	
	my @props;

		push @props, (
			{ SECTION_BEGIN => html('Topology Update') },
			{ SECTION => [
				{	INACTIVE => (GetKey($table, 'desktop_update_frequency') eq '0'),
					CP3 => 1,
					OBJ_VAL1 => $CheckTick,
       				OBJ_VAL2 => html('Update topology every'),
       				OBJ_VAL3 => html(int(GetKey($table, 'desktop_update_frequency')/3600)),
       				OBJ_VAL4 => html('Hours') },
				{	INACTIVE => (GetKey($table, 'desktop_update_frequency') ne '0'),
					OBJ_VAL1 => $Check,
       				OBJ_VAL2 => html('Update topology') }
       		] }
       	);
       	if (GetKey($table, 'desktop_update_frequency') ne '0') {
       		push @props, (
				{ SECTION => [
					{ 	OBJ_VAL1 => $Spacer,
						OBJ_VAL2 => (GetKey($table, 'desktop_silent_topo_update') eq 'true')?$MarkDot:$Mark,
						OBJ_VAL3 => html('Automatic update') },
					{	OBJ_VAL1 => $Spacer,
						OBJ_VAL2 => (GetKey($table, 'desktop_silent_topo_update') eq 'true')?$Mark:$MarkDot,
						OBJ_VAL3 => html('Upon VPN-1 SecuRemote/SecureClient start up') }
				] },
			);
		}
		
   		push @props, (
			{ SECTION_END => 1 }
		);
			
		push @props, (
			{ SECTION_BEGIN => html('Authentication Timeout') },
			{ SECTION => [
				{	CP3 => 1,
					OBJ_VAL1 => (GetKey($table, 'desktop_to_expire_passwords') eq 'false')?$MarkDot:$Mark,
    				OBJ_VAL2 => html('Use default value (1 day).') },
				 {	INACTIVE => (GetKey($table, 'desktop_to_expire_passwords') eq 'false'),
				 	CP3 => 1,
					OBJ_VAL1 => $MarkDot,
					OBJ_VAL2 => html('Validation timeout every'),
					OBJ_VAL3 => html(GetKey($table, 'desktop_password_expiry')),
					OBJ_VAL4 => html('Minutes') },
				 {	INACTIVE => (GetKey($table, 'desktop_to_expire_passwords') eq 'true'),
					OBJ_VAL1 => $Mark,
					OBJ_VAL2 => html('Validation timeout') }
			] },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($table, 'desktop_cache_fwz_passwords') eq 'true')?$CheckTick:$Check, 
    				OBJ_VAL2 => html('Allow caching of static passwords on client') }
    		] },
			{ SECTION_END => 1 }
    	);

		push @props, (
			{ SECTION_BEGIN => html('Additional Properties') },
			{ SECTION => [
				{	CP3 => 1,
					OBJ_VAL1 => (GetKey($table, 'desktop_keep_alive') eq 'true')?$CheckTick:$Check, 
   					OBJ_VAL2 => html('Enable Back Connections (from gateway to client)') }
   			] },
			{ SECTION => [
				{	INACTIVE => (GetKey($table, 'desktop_keep_alive') eq 'false'),
					CP3 => 1,
					OBJ_VAL1 => $Spacer,
        			OBJ_VAL2 => html('Send keep-alive packet to the Gateway every'),
        			OBJ_VAL3 => html(GetKey($table, 'desktop_keep_alive_interval')),
        			OBJ_VAL4 => html('Seconds') }
        	] },
			{ SECTION => [
				{	CP3 => 1,
					OBJ_VAL1 => (GetKey($table, 'encrypt_dns') eq 'true')?$CheckTick:$Check, 
    				OBJ_VAL2 => html('Encrypt DNS traffic') }
    		] },
  			{ SECTION_END => 1 }
    	);

		push @props, (
			{ SECTION_BEGIN => html('VPN-1 SecureClient - Logon High Availability') },
   			{ SECTION => [
   				{	CP3 => 1,
					OBJ_VAL1 => (GetKey($table, 'desktop_PS_HA') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Use backup Policy Servers on Logon failure (Transparent Mode only)') }
			] },
			{ SECTION => [
   				{	INACTIVE => (GetKey($table, 'desktop_PS_HA') eq 'false'),
   					OBJ_VAL1 => $Spacer,
					OBJ_VAL2 => (GetKey($table, 'desktop_PS_LB') eq 'false')?$MarkDot:$Mark,
					OBJ_VAL3 => html('Choose next Policy Server') },
   				{	INACTIVE => (GetKey($table, 'desktop_PS_HA') eq 'false'),
   					OBJ_VAL1 => $Spacer,
					OBJ_VAL2 => (GetKey($table, 'desktop_PS_LB') eq 'true')?$MarkDot:$Mark,
					OBJ_VAL3 => html('Choose Policy Server randomly') }
			] },
  			{ SECTION_END => 1 }
		);
 
		push @props, (
			{ SECTION_BEGIN => html('VPN-1 SecureClient - Desktop Security Policy expiration timer') },
			{ SECTION => [
				{	CP2 => 1, 	
					OBJ_VAL1 => html('Revert to default policy after'),
        			OBJ_VAL2 => html(GetKey($table, 'desktop_policy_expire')),
        			OBJ_VAL3 => html('Minutes') }
        	] },
  			{ SECTION_END => 1 }
        );

	$class{GP_PROPS}=\@props;
		
	push @classes, \%class;

	# ----------------------------------
	# CLASS: Remote Access - VPN Basic
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Remote Access';
	$class{GP_SUBCLASS}='VPN Basic';
	$class{GP_CLASS}='RA-VPNBasic';
	
	my @props;
		push @props, (
			{ SECTION_BEGIN => html('Support authentication methods') },
   			{ SECTION => [
   				{	OBJ_VAL1 => (GetKey($table, 'desktop_ike_preshared_support') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Pre-Shared Secret (for SecuRemote/SecureClient users)') },
   				{	# Below setting is unknown yet..... appears always to be true !!!
					#OBJ_VAL1 => (GetKey($table, '') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL1 => $CheckTick,
					OBJ_VAL2 => html('Public Key Signatures') },
   				{	INACTIVE => ($version ge '6'),
   					OBJ_VAL1 => (GetKey($table, 'desktop_ike_hybrid_support') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Hybrid Mode (VPN-1 & Firewall-1 authentication)') },
   				{	INACTIVE => ($version lt '6'),
   					OBJ_VAL1 => (GetKey($table, 'desktop_ike_hybrid_support') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Support Legacy Authentication for SC (hybrid mode), L2TP (PAP), and Nokia;').
								html('clients (CRACK)') }
			] },
			{ SECTION_END => 1 }
		);
	
		push @props, (
			{ SECTION_BEGIN => html('IKE over TCP') },
   			{ SECTION => [
   				{	CP3 => 1,
					OBJ_VAL1 => (GetKey($table, 'desktop_site_default_tcp_ike') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Gateway support IKE over TCP') }
			] },
			{ SECTION_END => 1 }
		);

		push @props, (
			{ SECTION_BEGIN => html('IP compression') },
   			{ SECTION => [
   				{	CP3 => 1,
					OBJ_VAL1 => (GetKey($table, 'desktop_ike_ipcomp_support') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Enable IP compression for SecureClient') }
			] },
			{ SECTION_END => 1 }
		);

		push @props, (
			{ SECTION_BEGIN => html('Load distribution') },
   			{ SECTION => [
   				{	CP3 => 1,
					OBJ_VAL1 => (GetKey($table, 'vpn_peer_ls') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Enable load distribution for Multiple Entry Points configurations (Remote;').
								html('Access connections)') }
			] },
			{ SECTION_END => 1 }
		);

		push @props, (
			{ SECTION_BEGIN => html('Nokia Clients') },
   			{ SECTION => [
   				{	CP3 => 1,
					OBJ_VAL1 => (GetKey($table, 'ike_support_nokia_crack') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Support remote access VPN using Nokia clients') }
			] },
			{ SECTION_END => 1 }
		);

	$class{GP_PROPS}=\@props;
		
	push @classes, \%class;

	# ----------------------------------
	# CLASS: Remote Access - VPN Advanced
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Remote Access';
	$class{GP_SUBCLASS}='VPN Advanced';
	$class{GP_CLASS}='RA-VPNAdv';
	
	my @props;
	if ($version lt '6') {	
		push @props, (
			{ SECTION_BEGIN => html('User Encryption Properties') },
			{ SECTION => [
				{ 	CP2 => 1,
					OBJ_VAL1 => html('Encryption Algorithm:'),
        			OBJ_VAL2 => html(GetKey($table, 'isakmp.encryption')) },
				{ 	CP2 => 1,
				 	OBJ_VAL1 => html('Data Integrity:'),
        			OBJ_VAL2 => html(GetKey($table, 'isakmp.data.integrity')) }
        	] },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($table, 'force_encryption_on_all_users') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Enforce Encryption Algorithm and Data Integrity on all users') },
				{	OBJ_VAL1 => $Spacer, 
					OBJ_VAL2 => html('Note: This global enforcement applies to NG FP2 and higher Modules,'.';').
								html('and overrides user specific Encryption Algorithm and Data Integrity settings') }
			] },
			{ SECTION_END => 1 }
		);

		# Find Diffie-Helman supported groups ...
		my @section;
		my @groups;
		foreach $var (GetMembers(GetKey($table, 'desktop_ike_p1_supported_dh_groups'))) {
			$val=GetKey(GetKey($var,'Table'), GetKey($var,'Name'));
			$groups[GetKey($val, 'DH_group_number')]=GetKey($var, 'Name');
		}
		push @props, (
			{ SECTION_BEGIN => html('IKE Security assocation Properties') },
			{ SECTION => [
				{ 	OBJ_VAL1 => html('Support Diffie-Hellman groups:'),
					OBJ_VAL2 => ((defined($groups[1]))?$CheckTick:$Check).html(' Group 1 (768 bit)') },
				{ 	OBJ_VAL1 => html(''),
					OBJ_VAL2 => ((defined($groups[2]))?$CheckTick:$Check).html(' Group 2 (1024 bit)') },
				{ 	OBJ_VAL1 => html(''),
					OBJ_VAL2 => ((defined($groups[5]))?$CheckTick:$Check).html(' Group 5 (1536 bit)') },
				{ 	OBJ_VAL1 => '' },
				{	CP2 => 1,
					OBJ_VAL1 => html('Use Diffie-Hellman group:'),
        			OBJ_VAL2 => html(stripquotes(GetKey($table, 'desktop_ike_phase1_use_DH_group:Name'))) }
        	] },
			{ SECTION_END => 1 }
        );
			
		push @props, (
			{ SECTION_BEGIN => html('Resolving mechanism') },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($table, 'resolve_interface_ranges') eq 'true')?$MarkDot:$Mark, 
					OBJ_VAL2 => html('Enable SecuRemote/SecureClient to calculate statically peer;').
								html("gateway's best interface based on network topology") },
				{	OBJ_VAL1 => (GetKey($table, 'resolve_interface_ranges') eq 'false')?$MarkDot:$Mark, 
					OBJ_VAL2 => html('Enable dynamic interface resolving by SecuRemote/SecureClient peers.;').
								html('must be defined per gateway)') }
			] },
			{ SECTION_END => 1 }
		);
	}

	push @props, (
		{ SECTION_BEGIN => html('SecuRemote/SecureClient behaviour while disconnected') },
		{ TXT => [{	CLASS => 'note_xspc', 
					TEXT => html('When disconnected, traffic to the encryption domain, will be') }] },
		{ SECTION => [
			{	OBJ_VAL1 => $Spacer,
				OBJ_VAL2 => (GetKey($table, 'allow_clear_traffic_while_disconnected') eq 'false')?$MarkDot:$Mark, 
				OBJ_VAL3 => html('Dropped') },
			{	OBJ_VAL1 => $Spacer,
				OBJ_VAL2 => (GetKey($table, 'allow_clear_traffic_while_disconnected') eq 'true')?$MarkDot:$Mark, 
				OBJ_VAL3 => html('Sent in clear') }
		] },
		{ SECTION_END => 1 }
	);

	if ($version ge '6') {
		push @props, (
			{ SECTION_BEGIN => html('Hub mode for SecureClient NG AI R56 and above in compact view') },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($table, 'simplified_client_route_all_traffic') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Route all traffic when connected') }
			] },
			{ SECTION_END => 1 }
		);
		
		push @props, (
			{ SECTION_BEGIN => html('Other settings for SecureClient NG AI R56 and above') },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($table, 'disconnect_when_in_enc_domain') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Disconnect client when getting an IP from a VPN domain') },
				{	OBJ_VAL1 => (GetKey($table, 'enable_log_collection') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Allow users to save troubleshooting logs') }
			] },
			{ SECTION_END => 1 }
		);

		push @props, (
			{ SECTION_BEGIN => html('Office Mode') },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($table, 'single_om_per_site') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Use first allocated Office Mode IP Address for all connections;').
								html('to the Gateways of the site') }
			] },
			{ SECTION_END => 1 }
		);
	}
		
	$class{GP_PROPS}=\@props;
		
	push @classes, \%class;

	# ----------------------------------
	# CLASS: Remote Access - IKE (Phase 1)
	# ----------------------------------
	if ($version ge '6') {
    	my %class;
    	$class{GP_MAINCLASS}='Remote Access';
    	$class{GP_SUBCLASS}='IKE (Phase 1)';
    	$class{GP_CLASS}='RA-IKE';
    	
    	my @props;

		# Find supported encryption algorithms ...
   		my @enc;
		foreach $var (GetMembers(GetKey($table, 'desktop_ike_p1_enc_algs'))) {
			$enc[$var] = 1;
		}
		# Find supported data integrity algorithms ...
   		my @data;
		foreach $var (GetMembers(GetKey($table, 'desktop_ike_p1_hash_algs'))) {
			$data[$var] = 1;
		}
		# Find supported Diffie-Hellman groups ...
   		my @groups;
		foreach $var (GetMembers(GetKey($table, 'desktop_ike_p1_supported_dh_groups'))) {
			$val=GetKey(GetKey($var,'Table'), GetKey($var,'Name'));
			$groups[GetKey($val, 'DH_group_number')]=GetKey($var, 'Name');
		}
		push @props, (
			{ SECTION => [
				{ 	OBJ_VAL1 => html('Support encryption algorithms:'),
					OBJ_VAL2 => ((defined($enc["DES"]))?$CheckTick:$Check).html(' DES') },
				{ 	OBJ_VAL1 => '',
					OBJ_VAL2 => ((defined($enc["3DES"]))?$CheckTick:$Check).html(' 3DES') },
				{ 	OBJ_VAL1 => '',
					OBJ_VAL2 => ((defined($enc["AES-128"]))?$CheckTick:$Check).html(' AES-128') },
				{ 	OBJ_VAL1 => '',
					OBJ_VAL2 => ((defined($enc["AES-256"]))?$CheckTick:$Check).html(' AES-256') },
				{ 	OBJ_VAL1 => '' },
				{	CP2 => 1,
					OBJ_VAL1 => html('Use encryption algorithm:'),
        			OBJ_VAL2 => html(stripquotes(GetKey($table, 'desktop_ike_phase1_use_enc'))) },
				{ 	OBJ_VAL1 => '' },

				{ 	OBJ_VAL1 => html('Support Data Integrity:'),
					OBJ_VAL2 => ((defined($data["MD5"]))?$CheckTick:$Check).html(' MD5') },
				{ 	OBJ_VAL1 => '',
					OBJ_VAL2 => ((defined($data["SHA1"]))?$CheckTick:$Check).html(' SHA1') },
				{ 	OBJ_VAL1 => '' },
				{	CP2 => 1,
					OBJ_VAL1 => html('Use encryption algorithm:'),
        			OBJ_VAL2 => html(stripquotes(GetKey($table, 'desktop_ike_phase1_use_hash'))) },
				{ 	OBJ_VAL1 => '' },

				{ 	OBJ_VAL1 => html('Support Diffie-Hellman groups:'),
					OBJ_VAL2 => ((defined($groups[1]))?$CheckTick:$Check).html(' Group 1 (768 bit)') },
				{ 	OBJ_VAL1 => html(''),
					OBJ_VAL2 => ((defined($groups[2]))?$CheckTick:$Check).html(' Group 2 (1024 bit)') },
				{ 	OBJ_VAL1 => html(''),
					OBJ_VAL2 => ((defined($groups[5]))?$CheckTick:$Check).html(' Group 5 (1536 bit)') },
				{ 	OBJ_VAL1 => html(''),
					OBJ_VAL2 => ((defined($groups[14]))?$CheckTick:$Check).html(' Group 14 (2048 bit)') },
				{ 	OBJ_VAL1 => '' },
				{	CP2 => 1,
					OBJ_VAL1 => html('Use Diffie-Hellman group:'),
        			OBJ_VAL2 => html(stripquotes(GetKey($table, 'desktop_ike_phase1_use_DH_group:Name'))) },
        	] }
        );

    	$class{GP_PROPS}=\@props;
    		
    	push @classes, \%class;
    }

	# ----------------------------------
	# CLASS: Remote Access - IPSEC (Phase 2)
	# ----------------------------------
	if ($version ge '6') {
    	my %class;
    	$class{GP_MAINCLASS}='Remote Access';
    	$class{GP_SUBCLASS}='IPSEC (Phase 2)';
    	$class{GP_CLASS}='RA-IPSEC';
    	
    	my @props;
		push @props, (
			{ SECTION_BEGIN => html('User Encryption Properties') },
			{ SECTION => [
				{	CP2 => 1,
					OBJ_VAL1 => html('Encryption Algorithm:'), 
					OBJ_VAL2 => html(stripquotes(GetKey($table, 'isakmp.encryption'))) },
				{	CP2 => 1,
					OBJ_VAL1 => html('Data Integrity:'), 
					OBJ_VAL2 => html(stripquotes(GetKey($table, 'isakmp.data.integrity'))) },
			] },
			{ SECTION => [
				{	OBJ_VAL1 => '' },
				{	OBJ_VAL1 => (GetKey($table, 'force_encryption_on_all_users') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html('Enforce Encryption Algorithm and Data Integrity on all users.') },
				{	OBJ_VAL1 => '',
					OBJ_VAL2 => html('Note: This global enforcement applies to NG FP2 and higher Modules,;').
								html('and overrides user specific Encryption Algorithm and Data Integrity settings.') },
			] },
			{ SECTION_END => 1 }
		);

    	$class{GP_PROPS}=\@props;
    		
    	push @classes, \%class;
    }

	# ----------------------------------
	# CLASS: Remote Access - Certificates
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Remote Access';
	$class{GP_SUBCLASS}='Certificates';
	$class{GP_CLASS}='RA-Cert';
	
	my @props;

		push @props, (
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($table, 'sr_dont_check_crl') eq 'false')?$CheckTick:$Check, 
					OBJ_VAL2 => html("Client will verify gateway's certificate against revocation list") },
				{	OBJ_VAL1 => $Spacer },
				{	OBJ_VAL1 => (GetKey($table, 'renew_users_ica_cert') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html("Renew users internal CA certificates") }
			] },
			{ SECTION => [
				{	INACTIVE => (GetKey($table, 'renew_users_ica_cert') eq 'false'),
					CP3 => 1,
					OBJ_VAL1 => $Spacer,
					OBJ_VAL2 => html('Start renewal process'),
        			OBJ_VAL3 => html(GetKey($table, 'desktop_policy_expire')),
        			OBJ_VAL4 => html('days before expiration date') }
			] }
		);

	$class{GP_PROPS}=\@props;
		
	push @classes, \%class;

	# ----------------------------------
	# CLASS: Remote Access - SCV
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Remote Access';
	$class{GP_SUBCLASS}='Secure Configuration Verification (SCV)';
	$class{GP_CLASS}='RA-SCV';
	
	my @props;

	push @props, (
		{ SECTION_BEGIN => html('Gateway SCV Options') },
		{ SECTION => [
			{	OBJ_VAL1 => (GetKey($table, 'enforce_desktop_config') eq 'true')?$CheckTick:$Check, 
				OBJ_VAL2 => html("Apply SCV on Simplified Mode Security Policies") },
		] }
	);
	# The exceptions to this are introduced in version 6 (NGX) ...
	if ($version ge '6') {
    	if (GetKey($table, 'enforce_desktop_config') eq 'true') {
    		my @table;
    		my $i = 0;
    		push @table, {
       			HEAD => 1,
       			OBJ_VAL1 => "",
       			OBJ_VAL2 => 'Hosts',
       			OBJ_VAL3 => 'Services' 
       		};
    
    		foreach $v1 (GetMembers(GetKey($table, 'non_scv_verified_hosts'))) {
    			my $server;
    			my $service;
    			$i++;
    			foreach $v2 (GetMembers(GetKey($v1, 'hosts'))) {
    				%var = %{obj2html('_GLOBAL', GetKey($v2, 'Table'), GetKey($v2, 'Name'), $_HREF)};
    				$server .= $var{OBJ_ICON}.' '.$var{OBJ_TEXT}.'<BR/>';
    			}
    			foreach $v2 (GetMembers(GetKey($v1, 'services'))) {
    				%var = %{obj2html('_GLOBAL', GetKey($v2, 'Table'), GetKey($v2, 'Name'), $_HREF)};
    				$service .= $var{OBJ_ICON}.' '.$var{OBJ_TEXT}.'<BR/>';
    			}
    			push @table, { 	
    			 	OBJ_VAL1 => $i,
    			 	OBJ_VAL2 => $server,
    			 	OBJ_VAL3 => $service
    			}
    		};
    
    		# Create Exceptions folded section, including table with servers and services ...
       		push @props, (
       			{ SF_BEG => [
       				{	SECTION  => html('Exceptions'),
       					BUTTON   => 1,
       					OBJ_NAME => $class{GP_CLASS},
       					STATE	 => 0 }
       			] },
    			{ TABLE => \@table },
    			{ SF_END => 1 }
    		);
        }
    }

	push @props, { SECTION_END => 1 };

		push @props, (
			{ SECTION_BEGIN => html('Upon verification failure:') },
			{ SECTION => [
				{	CP3 => 1,
					OBJ_VAL1 => (GetKey($table, 'scv_gw_verify_only_mode') eq 'false')?$MarkDot:$Mark, 
					OBJ_VAL2 => html("Block client's connection") },
				{	CP3 => 1,
					OBJ_VAL1 => (GetKey($table, 'scv_gw_verify_only_mode') eq 'true')?$MarkDot:$Mark, 
					OBJ_VAL2 => html("Accept and log client's connection") }
			] },
			{ SECTION_END => 1 }
		);

		push @props, (
			{ SECTION_BEGIN => html("Basic configuration verification on client's machine") },
			{ SECTION => [
				{	CP3 => 1,
					OBJ_VAL1 => (GetKey($table, 'desktop_security_protect_all_ifc') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html("Policy is installed on all interfaces") },
				{	CP3 => 1,
					OBJ_VAL1 => (GetKey($table, 'desktop_security_non_ip_protocols') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html("Only TCP/IP protocols are used") }
			] },
			{ SECTION_END => 1 }
		);

		push @props, (
			{ SECTION_BEGIN => html("Configuration Violation Notification on client's machine") },
			{ SECTION => [
				{	CP3 => 1,
					OBJ_VAL1 => (GetKey($table, 'desktop_security_send_log') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html("Generate log") },
				{	CP3 => 1,
					OBJ_VAL1 => (GetKey($table, 'desktop_security_send_warning') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html("Notify the user") }
			] },
			{ SECTION_END => 1 }
		);


	$class{GP_PROPS}=\@props;
		
	push @classes, \%class;

	# ----------------------------------
	# CLASS: Remote Access - SSL Network Extender 
	# ----------------------------------
	if ($version ge '6') {
    	my %class;
    	$class{GP_MAINCLASS}='Remote Access';
    	$class{GP_SUBCLASS}='SSL Network Extender';
    	$class{GP_CLASS}='RA-SSL';
    	
    	my @props;
    	push @props, (
    		{ SECTION => [
    			{	CP2 => 1, 	
    				OBJ_VAL1 => html('User Authentication method:'),
           			OBJ_VAL2 => html($snx_user_auth_methods{GetKey($table, 'snx_user_auth_methods')}) },
    			{	CP2 => 1, 	
    				OBJ_VAL1 => html('Client upgrade mode:'),
           			OBJ_VAL2 => html($snx_upgrade{GetKey($table, 'snx_upgrade')}) },
    			{	CP2 => 1, 	
    				OBJ_VAL1 => html('Supported encryption methods:'),
           			OBJ_VAL2 => html($snx_encryption_methods{GetKey($table, 'snx_encryption_methods')}) },
    			{	CP2 => 1, 	
    				OBJ_VAL1 => html('When client disconnects:'),
           			OBJ_VAL2 => html($snx_uninstall_on_disconnect{GetKey($table, 'snx_uninstall_on_disconnect')}) },
    			{	CP2 => 1, 	
    				OBJ_VAL1 => html('Integrity Clientless Security (ICS):'),
           			OBJ_VAL2 => html($snx_secure_conf_verification{GetKey($table, 'snx_secure_conf_verification')}) },
           	] }
        );
        
   		# Create advanced section ...
   		push @props, (
   			{ SF_BEG => [
   				{	SECTION  => html('Advanced'),
   					BUTTON   => 1,
   					OBJ_NAME => $class{GP_CLASS},
   					STATE	 => 0 }
   			] },
			{ SECTION => [
    			{	CP2 => 1, 	
    				OBJ_VAL1 => html('Re-authenticate user every'),
           			OBJ_VAL2 => GetKey($table, 'snx_user_re_auth_timeout'),
           			OBJ_VAL3 => html('minutes') },
    			{	CP2 => 1, 	
    				OBJ_VAL1 => html('Client sends keep-alive packets every'),
           			OBJ_VAL2 => GetKey($table, 'snx_keep_alive_timeout'),
           			OBJ_VAL3 => html('seconds') },
           	] },
			{ SF_END => 1 }
  		);

    	$class{GP_PROPS}=\@props;
    		
    	push @classes, \%class;
    }
				
	# ----------------------------------
	# CLASS: Remote Access - Early Version Compatibility
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Remote Access';
	$class{GP_SUBCLASS}='Early Version Compatibility';
	$class{GP_CLASS}='RA-EVC';
	
	my @props;
	push @props, (
		{ TXT => [
			{	CLASS => 'note',
				TEXT =>	html('The following settings apply only to VPN-1 SecureClient prior to NG.;').
						html('For later versions, these settings are configured in the Desktop Security policy.') }
		]}
	);
	push @props, (
		{ SECTION => [
			{	CP2 => 1, 	
				OBJ_VAL1 => html('Required policy for all desktops:'),
   				OBJ_VAL2 => html($EarlySCV{GetKey($table, 'desktop_security_policy_code')}) }
   		]}
   	);
	push @props, (
		{ SECTION => [
			{	OBJ_VAL1 => (GetKey($table, 'desktop_security_policy_installed') eq 'true')?$CheckTick:$Check, 
				OBJ_VAL2 => html("Client is enforcing required policy") }
		]}
	);

	# Section added in NGX ...
	if ($version gt '6') {
		push @props, (
			{ TXT => [
				{	CLASS => 'note',
					TEXT =>	html(";SecuRemote/SecureClient chooses peer gateway's interface using:") }
			]},
			{ SECTION => [
				{	OBJ_VAL1 => $Spacer,
					OBJ_VAL2 => (GetKey($table, 'resolve_interface_ranges') eq 'true')?$MarkDot:$Mark,
					OBJ_VAL3 => html("Static calculation based on network topology") },
				{	OBJ_VAL1 => $Spacer,
					OBJ_VAL2 => (GetKey($table, 'resolve_interface_ranges') eq 'false')?$MarkDot:$Mark,
					OBJ_VAL3 => html("Dynamic interface resolving mechanism") }
		]}

		);
	}	

	$class{GP_PROPS}=\@props;
		
	push @classes, \%class;
				
	# ----------------------------------
	# CLASS: Remote Access - Hot Spot/Hotel Registration
	# ----------------------------------
	if ($version gt '6') {
    	my %class;
    	$class{GP_MAINCLASS}='Remote Access';
    	$class{GP_SUBCLASS}='Hot Spot/Hotel Registration';
    	$class{GP_CLASS}='RA-Hot';
    	
    	my @props;
    
    	push @props, (
    		{ SECTION_BEGIN => html("Wireless Hot Spot and Hotel Inernet access registration") },
    		{ SECTION => [
    			{	OBJ_VAL1 => (GetKey($table, 'registration:enabled') eq 'true')?$CheckTick:$Check, 
    				OBJ_VAL2 => html("Enable registration") },
    		] },
    	);

		if (GetKey($table, 'registration:enabled') eq 'true') {  				

        	$v1 = join(";", GetMembers(GetKey($table, 'registration:ports')));
        	push @props, (
				{ SECTION => [
        			{	OBJ_VAL1 => $Spacer,
        				OBJ_VAL2 => ((GetKey($table, 'registration:local_subnets') eq 'true')?$CheckTick:$Check).
        							html(" Local subnets access only") },
        			{	CP3 => 1,
        				OBJ_VAL1 => "",
        				OBJ_VAL2 => html("Track:"),
            			OBJ_VAL3 => (GetKey($table, 'registration:log') eq 'true')?"Log":"None" },
        			{	CP3 => 1,
        				OBJ_VAL1 => "",
        				OBJ_VAL2 => html("Maximum time to complete registration:"),
            			OBJ_VAL3 => html(GetKey($table, 'registration:connect_timeout')),
            			OBJ_VAL4 => html("seconds") },
        			{	CP3 => 1,
        				OBJ_VAL1 => "",
        				OBJ_VAL2 => html("Allow access to maximum of:"),
            			OBJ_VAL3 => html(GetKey($table, 'registration:max_ip_count')),
            			OBJ_VAL4 => html("addresses during registration") },
        		] },
				{ SECTION => [
        			{	OBJ_VAL1 => $Spacer,
        				OBJ_VAL2 => html(";Ports to be opened during registration (up to 10 ports):") },
        		] },
				{ SECTION => [
        			{	CP3 => 1,
        				OBJ_VAL1 => $Spacer,
        				OBJ_VAL2 => $Spacer.$Spacer,
        				OBJ_VAL3 => html($v1) },
        		] },
        	);
        	
        }

		push @props, { SECTION_END => 1 };    
		
    	$class{GP_PROPS}=\@props;
    		
    	push @classes, \%class;
    }

	# ----------------------------------
	# CLASS: Smart Directory (LDAP)
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Smart Directory (LDAP)';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='LDAP';
	
	my @props;

	push @props, (
		{ SECTION => [
			{	OBJ_VAL1 => (GetKey($table, 'fwldap_UseLDAP') eq 'true')?$CheckTick:$Check, 
				OBJ_VAL2 => html("Use SmartDirectory (LDAP)") }
		]}
	);
	
	if (GetKey($table, 'fwldap_UseLDAP') eq 'true') {
		push @props, (
			{ SECTION_BEGIN => html("SmartDirectory (LDAP) Properties:") },
			{ SECTION => [
				{	CP2 => 1, 	
					OBJ_VAL1 => html('Timeout on cached users:'),
    				OBJ_VAL2 => html(GetKey($table, 'fwldap_CacheTimeout')),
    				OBJ_VAL3 => html('seconds') },
				{	CP2 => 1, 	
					OBJ_VAL1 => html('Cache size:'),
    				OBJ_VAL2 => html(GetKey($table, 'fwldap_CacheSize')),
    				OBJ_VAL3 => html('users') },
				{	INACTIVE => (GetKey($table, 'fwldap_is_PasswordExpiration') eq 'false'),
					CP2 => 1, 	
					OBJ_VAL1 => $CheckTick.html(' Password expires after:'),
    				OBJ_VAL2 => html(GetKey($table, 'fwldap_PasswordExpiration')),
    				OBJ_VAL3 => html('days') },
				{	INACTIVE => (GetKey($table, 'fwldap_is_PasswordExpiration') eq 'true'),
					CP2 => 1, 	
					OBJ_VAL1 => $Check.html(' Password expires after:'),
    				OBJ_VAL2 => html('-'),
    				OBJ_VAL3 => html('days') },
    		] },
		);

		$var = stripquotes(GetKey($table, 'fwldap_DisplayDN'));
		push @props, (
			{ 	SECTION_BEGIN => html("Display User's DN at Login") },
			{	SECTION => [
					{	OBJ_VAL1 => (($var eq 'no display')?$MarkDot:$Mark).html(" Don't Display").$Spacer,
						OBJ_VAL2 => (($var eq 'display')?$MarkDot:$Mark).html(" Display").$Spacer,
						OBJ_VAL3 => (($var eq 'display upon request')?$MarkDot:$Mark).html(" Display on Request") }
			] },
			{	SECTION_END => 1 }
		);			

		push @props, (
			{ SECTION_BEGIN => html("Password Strength:") },
			{ SECTION => [
				{	CP2 => 1, 	
					OBJ_VAL1 => html('Minimal password length:'),
    				OBJ_VAL2 => html(GetKey($table, 'psswd_min_length')),
    				OBJ_VAL3 => html('characters') }
    		] },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($table, 'psswd_min_num_of_lowercase') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html("Password must include lowercase character") },
				 {	OBJ_VAL1 => (GetKey($table, 'psswd_min_num_of_uppercase') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html("Password must include an uppercase character") },
				 {	OBJ_VAL1 => (GetKey($table, 'psswd_min_num_of_numbers') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html("Password must include a digit") },
				 {	OBJ_VAL1 => (GetKey($table, 'psswd_min_num_of_symbols') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html("Password must include a symbol") },
				 {	OBJ_VAL1 => $Spacer },
				 {	OBJ_VAL1 => (GetKey($table, 'fwldap_ApplyPwdRulesOnMgmt') eq 'true')?$CheckTick:$Check, 
					OBJ_VAL2 => html("Enforce rules for user management administrators") }
			] },
			{ SECTION_END => 1 }
		);
		push @props, { SECTION_END => 1 },

	} # End IF for SmartDirectory (LDAP)

	$class{GP_PROPS}=\@props;

	push @classes, \%class;

	# ----------------------------------
	# CLASS: Floodgate-1 / QoS
	# ----------------------------------
	my %class;
	# This section is renamed in version NGX ...
	if ($version lt '6') {
		$class{GP_MAINCLASS}='Floodgate-1';
		$class{GP_SUBCLASS}='';
		$class{GP_CLASS}='Floodgate';
	} else {
		$class{GP_MAINCLASS}='QoS';
		$class{GP_SUBCLASS}='';
		$class{GP_CLASS}='QoS';
	}	

	$obj = GetKey($table, 'floodgate_preferences');
	
	my @props;
	push @props, (
		{ SECTION_BEGIN => html("Weight") },
		{ SECTION => [
			{	CP2 => 1, 	
				OBJ_VAL1 => html('Maximum weight of rule:'),
    			OBJ_VAL2 => html(GetKey($obj, 'default_max_weight')) },
			 {	CP2 => 1, 	
				OBJ_VAL1 => html('Default weight of rule:'),
    			OBJ_VAL2 => html(GetKey($obj, 'default_weight')),
    			OBJ_VAL3 => html('Note: Value will be applied to;new rules only') }
    	] },
    	{ SECTION_END => 1 },

		{ SECTION_BEGIN => html("Rate") },
		{ SECTION => [
			{	CP2 => 1, 	
				OBJ_VAL1 => html('Unit of measure:'),
    			OBJ_VAL2 => html($UnitsMeasure{GetKey($obj, 'rate_units')}) }
    	] },
    	{ SECTION_END => 1 },

		{ SECTION_BEGIN => html("Authentication timeout for QoS") },
		{ SECTION => [
			{	CP2 => 1, 	
				OBJ_VAL1 => html('Authenticated IP expires after'),
    			OBJ_VAL2 => html(GetKey($obj, 'auth_true_timeout')),
    			OBJ_VAL3 => html('minutes') },
			 {	CP2 => 1, 	
				OBJ_VAL1 => html('Non authenticated IP expires after'),
    			OBJ_VAL2 => html(GetKey($obj, 'auth_false_timeout')),
    			OBJ_VAL3 => html('minutes') },
			 {	CP2 => 1, 	
				OBJ_VAL1 => html('Unanswered queried IP expires after'),
    			OBJ_VAL2 => html(GetKey($obj, 'auth_query_timeout')),
    			OBJ_VAL3 => html('minutes') }
    	] },
    	{ SECTION_END => 1 },
	);

	$class{GP_PROPS}=\@props;

	push @classes, \%class;

	# ----------------------------------
	# CLASS: Firewall-1 GX
	# ----------------------------------
	if ($version lt '6') {
    	my %class;
    	$class{GP_MAINCLASS}='FireWall-1 GX';
    	$class{GP_SUBCLASS}='';
    	$class{GP_CLASS}='FW1-GX';
    	
    	my @props;
    
    		push @props, (
    			{ SECTION_BEGIN => html("GTP Intra Tunnel Inpspection") },
    			{ SECTION => [
    				{	CP1 => 1,
    					OBJ_VAL1 => (GetKey($table, 'gtp_anti_spoofing') eq 'true')?$CheckTick:$Check, 
    					OBJ_VAL2 => html("Enforce GTP anti spoofing") },
    				 {	CP1 => 1,
    					OBJ_VAL1 => (GetKey($table, 'block_gtp_in_gtp') eq 'true')?$CheckTick:$Check, 
    					OBJ_VAL2 => html("Block GTP in GTP") }
    			] },
		    	{ SECTION_END => 1 },
    		);
    
    		push @props, (
    			{ SECTION_BEGIN => html("Track") },
    			{ SECTION => [
    				{	CP2 => 1,
    					OBJ_VAL1 => ((GetKey($table, 'gtp_code_alert') eq 'true')?$CheckTick:$Check).
    								html(' Produce extended logs on unmatched PDUs'),
    					OBJ_VAL2 => (GetKey($table, 'gtp_code_alert') eq 'true')?
    								$Position{stripquotes(GetKey($table, 'gtp_code_alert_p'))}:'' },
    				{	CP2 => 1,
    					OBJ_VAL1 => html("Protocol violation track option"),
    					OBJ_VAL2 => html($Track{GetKey($table, 'gtp_track')}) }
    			] },
		    	{ SECTION_END => 1 },
    		);
    
    		$var = GetKey($table, 'allow_PDU_sequence');
    		$val = GetKey($table, 'PDU_sequence');
    		push @props, (
    			{ SECTION_BEGIN => html("GTP PDU Integrity Tests") },
    			{ TXT => [
    				{	CLASS => 'note_xspc',
    					TEXT => html('To enhance performance, disable one or more of the extended integrity tests below') }
    			] },
    			{ SECTION => [
    				{	CP1 => 1,
    					OBJ_VAL1 => (GetKey($table, 'check_flow_labels') eq 'true')?$CheckTick:$Check,
    					OBJ_VAL2 => html("Verify Flow Labels") },
    				 {	CP3 => 1,
    					OBJ_VAL1 => ($var eq 'true')?$CheckTick:$Check,
    					OBJ_VAL2 => html('G-PDU seq number check with a maximum deviation of '),
    					OBJ_VAL3 => ($var eq 'true')?$val:'' }
    			] },
		    	{ SECTION_END => 1 },
    		);
    
    		push @props, (
    			{ SECTION_BEGIN => html("Other") },
    			{ SECTION => [
    				{	CP1 => 1,
    					OBJ_VAL1 => (GetKey($table, 'gtp_sam_close_upon_delete') eq 'true')?$CheckTick:$Check,
    					OBJ_VAL2 => html("Send 'clean state' request on each GTP tunnel Deactivation") },
    				{	CP1 => 1,
    					OBJ_VAL1 => (GetKey($table, 'gtp_allow_multi_if_ggsn') eq 'true')?$CheckTick:$Check,
    					OBJ_VAL2 => html("Allow GGSN replies from multiple interfaces") },
    				{	CP1 => 1,
    					OBJ_VAL1 => (GetKey($table, 'enable_reverse_connections') eq 'true')?$CheckTick:$Check,
    					OBJ_VAL2 => html("Enable Reverse Connections") }
    			] },
    			{ SECTION => [
    				{	CP2 => 1,
    					OBJ_VAL1 => html("GTP Signaling rate limit sampling interval "),
    					OBJ_VAL2 => GetKey($table, 'rate_limit_sampling_interval'),
    					OBJ_VAL3 => html("seconds") },
    				{	CP2 => 1,
    					OBJ_VAL1 => html("Allow one GTP Echo on each path every "),
    					OBJ_VAL2 => GetKey($table, 'gtp_echo_frequency'),
    					OBJ_VAL3 => html("seconds") }
    			] },
		    	{ SECTION_END => 1 },
    		);
    
    	$class{GP_PROPS}=\@props;
    
    	push @classes, \%class;
    }

	# ----------------------------------
	# CLASS: UserAuthority
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='UserAuthority';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='UA';
	
	my @props;

	my @section;
	@var = GetMembers(GetKey($table, 'netso_selected_trusted_domains_list'));
	foreach $var (@var) {
		push @section, {	CP2 => 1,
							OBJ_VAL1 => $Spacer,
							OBJ_VAL2 => html($var) }
	}

	push @props, (
		{ SECTION_BEGIN => html("VPN-1 Pro/Express Trusted Domains") },
		{ TXT => [
			{	CLASS => 'note',
				TEXT => html('When matching VPN-1/FireWall-1 usernames to Windows Domains usernames for;Single Sign On:') }
		] },
		{ SECTION => [
			{	OBJ_VAL1 => (GetKey($table, 'netso_trust_win_domains') eq 'all')?$MarkDot:$Mark, 
				OBJ_VAL2 => html("Trust all Windows Domains") },
			{	OBJ_VAL1 => (GetKey($table, 'netso_trust_win_domains') eq 'selectively')?$MarkDot:$Mark, 
				OBJ_VAL2 => html("Trust only the following Windows Domains:") }
		]},
		{ SECTION => \@section },
    	{ SECTION_END => 1 },
	);

	$class{GP_PROPS}=\@props;

	push @classes, \%class;

	# ----------------------------------
	# CLASS: Management High Availability
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Management High Availability';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='MgmtHA';
	
	my @props;

	push @props, (
		{ TXT => [
			{	CLASS => 'note',
				TEXT => html('Select the method by which the SmartCenter Servers will be synchronized:') }
		] },
		{ SECTION => [
			{	CP1 => 1,
				OBJ_VAL1 => (GetKey($table, 'auto_sync_on_install') eq 'true')?$MarkDot:$Mark,
				OBJ_VAL2 =>	html('Automatic synchronization when policy is installed') },
		] }
	);
		
	if (GetKey($table, 'auto_sync_on_install') eq 'true') {
		$var = GetKey($table, 'secondary_sync_schedule');
		%var = %{obj2html("Global", GetKey($var, 'Table'), GetKey($var, 'Name'), $_HREF)};
		$var = $var{OBJ_ICON}.' '.$var{OBJ_TEXT};
		$v1 = (GetKey($table, 'auto_sync_on_save') eq 'true');
		$v2 = (GetKey($table, 'auto_sync_scheduled') eq 'true');
		push @props, (
			{ SECTION => [
				{	OBJ_VAL1 => $Spacer,
					OBJ_VAL2 => html('Additional events that activate automatic synchronization:') },
			] },
			{ SECTION => [
				{ 	OBJ_VAL1 => $Spacer,
					OBJ_VAL2 => ($v1?$CheckTick:$Check),
				 	OBJ_VAL3 => html('Every time a policy is saved') },
				{ 	OBJ_VAL1 => '',
					OBJ_VAL2 => ($v2?$CheckTick:$Check),
				 	OBJ_VAL3 => html('On scheduled event: '),
				 	OBJ_VAL4 => ($v2?$var:'') }
			] }
		);
	}

	push @props, (
		{ SECTION => [
			{	CP1 => 1,
				OBJ_VAL1 => (GetKey($table, 'auto_sync_on_install') eq 'false')?$MarkDot:$Mark,
				OBJ_VAL2 =>	html('Manual synchronization only') }
		] },
		{ SECTION => [
			{	OBJ_VAL1 => '' },
			{	CP2 => 1,
				OBJ_VAL1 => html('Type of notification for Management High Availability tracking '),
				OBJ_VAL2 =>	$Track{GetKey($table, 'mgmtha_alert_type')} }
		] }
	);
						 		
	$class{GP_PROPS}=\@props;

	push @classes, \%class;

	# ----------------------------------
	# CLASS: Connect Control
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='ConnectControl';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='CC';
	
	my @props;
		push @props, (
			{ SECTION_BEGIN => html("Servers Availability") },
			{ SECTION => [
				{	CP2 => 1,
					OBJ_VAL1 => html("Servers availability check interval: "),
					OBJ_VAL2 => GetKey($table, 'physical_server_availability_check_interval'),
					OBJ_VAL3 => html("seconds") },
				{	CP2 => 1,
					OBJ_VAL1 => html("Server check retries: "),
					OBJ_VAL2 => GetKey($table, 'physical_server_check_retries'),
					OBJ_VAL3 => html("times") }
			] },
			{ SECTION_END => 1 }
		);
		push @props, (
			{ SECTION_BEGIN => html("Servers Persistency") },
			{ SECTION => [
				{	CP2 => 1,
					OBJ_VAL1 => html("Persistent Server timeout: "),
					OBJ_VAL2 => GetKey($table, 'logical_servers_persistent_timeout'),
					OBJ_VAL3 => html("seconds") }
			] },
			{ SECTION_END => 1 }
		);
		push @props, (
			{ SECTION_BEGIN => html("Servers Load Balancing") },
			{ SECTION => [
				{	CP2 => 1,
					OBJ_VAL1 => html("Load agents port: "),
					OBJ_VAL2 => GetKey($table, 'load_service_port') },
				{	CP2 => 1,
					OBJ_VAL1 => html("Load measurement interval: "),
					OBJ_VAL2 => GetKey($table, 'lbalanced_load_period_wakeup_sec'),
					OBJ_VAL3 => html("seconds") }
			] },
			{ SECTION_END => 1 }
		);
						 		
	$class{GP_PROPS}=\@props;

	push @classes, \%class;

	# ----------------------------------
	# CLASS: OSE - Open Security Extension
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='OSE - Open Security Extension';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='OSE';
	
	my @props;
	$v1 = (GetKey($table, 'established_router') eq 'true');
	$v2 = (GetKey($table, 'rip_router') eq 'true');
	$v3 = (GetKey($table, 'domain_udp_router') eq 'true');
	$v4 = (GetKey($table, 'domain_tcp_router') eq 'true');
	$v5 = (GetKey($table, 'icmpenable_router') eq 'true');
	push @props, (
		{ SECTION_BEGIN => html("OSE Access List") },
		{ SECTION => [
			{	OBJ_VAL1 => '' },
			{	CP3 => 1,
				OBJ_VAL1 => $v1?$CheckTick:$Check,
				OBJ_VAL2 => html('Accept established TCP connections:'),
				OBJ_VAL3 => $v1?$Position{stripquotes(GetKey($table, 'established_router_p'))}:'' },
			{	OBJ_VAL1 => '' },
			{	CP3 => 1,
				OBJ_VAL1 => $v2?$CheckTick:$Check,
				OBJ_VAL2 => html('Accept RIP:'),
				OBJ_VAL3 => $v2?$Position{stripquotes(GetKey($table, 'rip_router_p'))}:'' },
			{	OBJ_VAL1 => '' },
			{	CP3 => 1,
				OBJ_VAL1 => $v3?$CheckTick:$Check,
				OBJ_VAL2 => html('Accept Domain Name over UDP (Queries):'),
				OBJ_VAL3 => $v3?$Position{stripquotes(GetKey($table, 'domain_udp_router_p'))}:'' },
			{	OBJ_VAL1 => '' },
			{	CP3 => 1,
				OBJ_VAL1 => $v4?$CheckTick:$Check,
				OBJ_VAL2 => html('Accept Domain Name over TCP (Zone Transfer):'),
				OBJ_VAL3 => $v4?$Position{stripquotes(GetKey($table, 'domain_tcp_router_p'))}:'' },
			{	OBJ_VAL1 => '' },
			{	CP3 => 1,
				OBJ_VAL1 => $v5?$CheckTick:$Check,
				OBJ_VAL2 => html('Accept ICMP requests:'),
				OBJ_VAL3 => $v5?$Position{stripquotes(GetKey($table, 'icmpenable_router_p'))}:'' }
		] }
	);

	$class{GP_PROPS}=\@props;

	push @classes, \%class;

	# ----------------------------------
	# CLASS: Stateful Inspection
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Stateful Inspection';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='StateInspect';
	
	my @props;
	push @props, (
		{ SECTION_BEGIN => html("Default Session Timeouts") },
		{ SECTION => [
			{	CP2 => 1,
				OBJ_VAL1 => html('TCP start timeout:'),
				OBJ_VAL2 => GetKey($table, 'tcpstarttimeout'),
				OBJ_VAL3 => html('seconds') },
			{	CP2 => 1,
				OBJ_VAL1 => html('TCP session timeout:'),
				OBJ_VAL2 => GetKey($table, 'tcptimeout'),
				OBJ_VAL3 => html('seconds') },
			{	CP2 => 1,
				OBJ_VAL1 => html('TCP end timeout:'),
				OBJ_VAL2 => GetKey($table, 'tcpendtimeout'),
				OBJ_VAL3 => html('seconds') },
			{	CP2 => 1,
				OBJ_VAL1 => html('UDP virtual session timeout:'),
				OBJ_VAL2 => GetKey($table, 'udptimeout'),
				OBJ_VAL3 => html('seconds') },
			{	CP2 => 1,
				OBJ_VAL1 => html('ICMP virtual session timeout:'),
				OBJ_VAL2 => GetKey($table, 'icmptimeout'),
				OBJ_VAL3 => html('seconds') },
			{	CP2 => 1,
				OBJ_VAL1 => html('Other IP protocols virtual session timeout:'),
				OBJ_VAL2 => GetKey($table, 'othertimeout'),
				OBJ_VAL3 => html('seconds') }
		] },
		{ SECTION_END => 1 }
	);

	push @props, (
		{ SECTION_BEGIN => html("Stateful UDP") },
		{ SECTION => [{	CP1 => 1,
					OBJ_VAL1 => (GetKey($table, 'udpreply') eq 'true')?$CheckTick:$Check,
					OBJ_VAL2 => html('Accept stateful UDP replies for unknown services') }
		] },
		{ SECTION => [
			{	INACTIVE => !(($version lt '6') && (GetKey($table, 'udpreply') eq 'true')),
				OBJ_VAL1 => (GetKey($table, 'udpreply_from_any_port') eq 'true')?$CheckTick:$Check,
				OBJ_VAL2 => html('Accept stateful UDP replies from any port for unknown services') }
		] },
		{ SECTION_END => 1 }
	);
	
	push @props, (
		{ SECTION_BEGIN => html("Accept Stateful ICMP") },
		{ SECTION => [
			{	OBJ_VAL1 => (GetKey($table, 'icmpreply') eq 'true')?$CheckTick:$Check,
				OBJ_VAL2 => html('Replies'),
				OBJ_VAL3 => (GetKey($table, 'icmperrors') eq 'true')?$CheckTick:$Check,
				OBJ_VAL4 => html('Errors') }
		] },
		{ SECTION_END => 1 }
	);
	
	push @props, (
		{ SECTION_BEGIN => html("Stateful Other IP Protocols") },
		{ SECTION => [
			{	OBJ_VAL1 => (GetKey($table, 'otherreply') eq 'true')?$CheckTick:$Check,
				OBJ_VAL2 => html('Accept stateful other IP protocols replies for unknown services') }
		] },
		{ SECTION_END => 1 }
	);

	$v1 = (GetKey($table, 'fw_allow_out_of_state_tcp') eq '0');
	$v2 = (GetKey($table, 'fw_drop_out_of_state_udp') eq 'true');
	$v3 = (GetKey($table, 'fw_drop_out_of_state_icmp') eq 'true');
	push @props, (
		{ SECTION_BEGIN => html("Out of state packets") },
		{ SECTION => [
			{	INACTIVE => (GetKey($table, 'fw_allow_out_of_state_tcp') eq '0'),
				OBJ_VAL1 => $Check,
				OBJ_VAL2 => html('Drop out of state TCP packets') },
			{	INACTIVE => (GetKey($table, 'fw_allow_out_of_state_tcp') ne '0'),
				OBJ_VAL1 => $CheckTick,
				OBJ_VAL2 => html('Drop out of state TCP packets'),
				OBJ_VAL3 => $Spacer.$Spacer.$Spacer,
				OBJ_VAL4 => $v1?((GetKey($table, 'fw_log_out_of_state_tcp') eq '1')?$CheckTick:$Check):'',
				OBJ_VAL5 => $v1?html('Log on drop'):'' },
			{	INACTIVE => (GetKey($table, 'fw_drop_out_of_state_udp') eq 'true'),
				OBJ_VAL1 => $Check,
				OBJ_VAL2 => html('Drop out of state UDP packets') },
			{	INACTIVE => (GetKey($table, 'fw_drop_out_of_state_udp') ne 'true'),
				OBJ_VAL1 => $CheckTick,
				OBJ_VAL2 => html('Drop out of state UDP packets'),
				OBJ_VAL3 => $Spacer.$Spacer.$Spacer,
				OBJ_VAL4 => (GetKey($table, 'fw_log_out_of_state_udp') eq '1')?$CheckTick:$Check,
				OBJ_VAL5 => html('Log on drop') },
			{	INACTIVE => (GetKey($table, 'fw_drop_out_of_state_icmp') eq 'true'),
				OBJ_VAL1 => $Check,
				OBJ_VAL2 => html('Drop out of state ICMP packets') },
			{	INACTIVE => (GetKey($table, 'fw_drop_out_of_state_icmp') ne 'true'),
				OBJ_VAL1 => $CheckTick,
				OBJ_VAL2 => html('Drop out of state ICMP packets'),
				OBJ_VAL3 => $Spacer.$Spacer.$Spacer,
				OBJ_VAL4 => (GetKey($table, 'fw_log_out_of_state_icmp') eq '1')?$CheckTick:$Check,
				OBJ_VAL5 => html('Log on drop') }
		] },
		{ SECTION_END => 1 }
	);

	$class{GP_PROPS}=\@props;

	push @classes, \%class;

	# ----------------------------------
	# CLASS: Log and Alert
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Log and Alert';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='LogAlert';
	
	my @props;
	push @props, (
		{ SECTION_BEGIN => html("Track Options") },
		{ SECTION => [
			{	CP2 => 1,
				OBJ_VAL1 => html('VPN successful key exchange:'),
				OBJ_VAL2 => $Track{GetKey($table, 'vpn_success_key_exch')} },
			{	CP2 => 1,
				OBJ_VAL1 => html('VPN packet handling errors:'),
				OBJ_VAL2 => $Track{GetKey($table, 'vpn_packet_handle_prob')} },
			{	CP2 => 1,
				OBJ_VAL1 => html('VPN configuration & key exchange errors:'),
				OBJ_VAL2 => $Track{GetKey($table, 'vpn_conf_n_key_exch_prob')} },
			{	CP2 => 1,
				OBJ_VAL1 => html('IP options drop:'),
				OBJ_VAL2 => $Track{GetKey($table, 'ipoptslog')} },
			{	CP2 => 1,
				OBJ_VAL1 => html('Administrative notifications:'),
				OBJ_VAL2 => $Track{GetKey($table, 'maintenance_notification')} },
			{	CP2 => 1,
				OBJ_VAL1 => html('SLA violation:'),
				OBJ_VAL2 => $Track{GetKey($table, 'e2e_sla_alert')} },
			{	CP2 => 1,
				OBJ_VAL1 => html('Connection matched by SAM:'),
				OBJ_VAL2 => $Track{GetKey($table, 'sam_track')} },
			{	CP2 => 1,
				OBJ_VAL1 => html('Dynamic object resolution failure:'),
				OBJ_VAL2 => $Track{GetKey($table, 'dynamic_objects_track')} }
		] },
		{ SECTION_END => 1 }
	);

	# Note: the log VoIP parameter is not in the objects_5_0.C file. Unknown
	# where to get this information otherwise...
	push @props, (
		{ SECTION_BEGIN => html("Logging Modifiers") },
		{ SECTION => [
			{	OBJ_VAL1 => (GetKey($table, 'http_log_every_connection') eq 'true')?$CheckTick:$Check,
				OBJ_VAL2 => html('Log every authenticated HTTP connection')	},
			{	OBJ_VAL1 => (GetKey($table, 'h323_log_conn') eq 'true')?$CheckTick:$Check,
			 	OBJ_VAL2 => html('Log VoIP connection') },
		] },
		{ SECTION_END => 1 }
	);
		
	push @props, (			
		{ SECTION_BEGIN => html("Community Default Rule") },
		{ SECTION => [
			{	CP2 => 1,
				OBJ_VAL1 => html('Log Traffic:'),
				OBJ_VAL2 => $Track{GetKey($table, 'log_default_rule')} }
		] }
	);

	$class{GP_PROPS}=\@props;

	push @classes, \%class;

	# ----------------------------------
	# CLASS: Log and Alert - Time Settings
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Log and Alert';
	$class{GP_SUBCLASS}='Time Settings';
	$class{GP_CLASS}='LogTime';
	
	my @props;
	push @props, (			
		{ SECTION => [
			{	CP2 => 1,
				OBJ_VAL1 => html('Excessive log grace period:'),
				OBJ_VAL2 => GetKey($table, 'loggrace'),
				OBJ_VAL3 => html('seconds') },
			{	CP2 => 1,
				OBJ_VAL1 => html('SmartView Tracker resolving timeout:'),
				OBJ_VAL2 => GetKey($table, 'pagetimeout'),
				OBJ_VAL3 => html('seconds') },
			{	CP2 => 1,
				OBJ_VAL1 => html('Virtual Links statistics logging interval:'),
				OBJ_VAL2 => GetKey($table, 'e2e_statistics_interval'),
				OBJ_VAL3 => html('seconds') },
			{	CP2 => 1,
				OBJ_VAL1 => html('Status fetching interval:'),
				OBJ_VAL2 => GetKey($table, 'status_mgr_interval_seconds'),
				OBJ_VAL3 => html('seconds') }
		] }
	);

	$class{GP_PROPS}=\@props;

	push @classes, \%class;

	# ----------------------------------
	# CLASS: Log and Alert - Alert Commands
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Log and Alert';
	$class{GP_SUBCLASS}='Alert Commands';
	$class{GP_CLASS}='LogAlertCmds';
	
	my @props;
	push @props, (
		{ SECTION => [
			{	OBJ_VAL1 => (GetKey($table, 'alertcmd_send_to_system_status') eq 'true')?$CheckTick:$Check,
				OBJ_VAL2 => html('Send popup alert to SmartView Monitor') },
			{	CP3 => 1,
				OBJ_VAL1 => (GetKey($table, 'alertcmd') ne '')?$CheckTick:$Check,
				OBJ_VAL2 => html('Run popup alert script:'),
				OBJ_VAL3 => html(stripquotes(GetKey($table, 'alertcmd'))) },
			{ 	OBJ_VAL1 => '' },
			{	OBJ_VAL1 => (GetKey($table, 'mailcmd_send_to_system_status') eq 'true')?$CheckTick:$Check,
				OBJ_VAL2 => html('Send mail alert to SmartView Monitor') },
			{	CP3 => 1,
				OBJ_VAL1 => (GetKey($table, 'mailcmd') ne '')?$CheckTick:$Check,
				OBJ_VAL2 => html('Run mail alert script:'),
				OBJ_VAL3 => html(stripquotes(GetKey($table, 'mailcmd'))) },
			{ 	OBJ_VAL1 => '' },
			{	OBJ_VAL1 => (GetKey($table, 'snmptrapcmd_send_to_system_status') eq 'true')?$CheckTick:$Check,
				OBJ_VAL2 => html('Send SNMP trap alert to SmartView Monitor') },
			{	CP3 => 1,
				OBJ_VAL1 => (GetKey($table, 'snmptrapcmd') ne '')?$CheckTick:$Check,
				OBJ_VAL2 => html('Run SNMP trap alert script:'),
				OBJ_VAL3 => html(stripquotes(GetKey($table, 'snmptrapcmd'))) },
			{ 	OBJ_VAL1 => '' },
			{	OBJ_VAL1 => (GetKey($table, 'useralertcmd_send_to_system_status') eq 'true')?$CheckTick:$Check,
				OBJ_VAL2 => html('Send user defined alert no.1 to SmartView Monitor') },
			{	CP3 => 1,
				OBJ_VAL1 => (GetKey($table, 'useralertcmd') ne '')?$CheckTick:$Check,
				OBJ_VAL2 => html('Run UserDefined script:'),
				OBJ_VAL3 => html(stripquotes(GetKey($table, 'useralertcmd'))) },
			{ 	OBJ_VAL1 => '' },
			{	OBJ_VAL1 => (GetKey($table, 'useralert2cmd_send_to_system_status') eq 'true')?$CheckTick:$Check,
				OBJ_VAL2 => html('Send user defined alert no.2 to SmartView Monitor') },
			{	CP3 => 1,
				OBJ_VAL1 => (GetKey($table, 'useralert2cmd') ne '')?$CheckTick:$Check,
				OBJ_VAL2 => html('Run UserDefined script:'),
				OBJ_VAL3 => html(stripquotes(GetKey($table, 'useralert2cmd'))) },
			{ 	OBJ_VAL1 => '' },
			{	OBJ_VAL1 => (GetKey($table, 'useralert3cmd_send_to_system_status') eq 'true')?$CheckTick:$Check,
				OBJ_VAL2 => html('Send user defined alert no.3 to SmartView Monitor') },
			{	CP3 => 1,
				OBJ_VAL1 => (GetKey($table, 'useralert3cmd') ne '')?$CheckTick:$Check,
				OBJ_VAL2 => html('Run UserDefined script:'),
				OBJ_VAL3 => html(stripquotes(GetKey($table, 'useralert3cmd'))) },
			{ 	OBJ_VAL1 => '' },
		] }
	);

	# Support for 4.1 has been removed in NGX ...
	if ($version lt '6') {
    	push @props, (
    		{ SECTION_BEGIN => html('Early Versions Compatibility') },
    		{ SECTION => [
    			{	OBJ_VAL1 => (GetKey($table, 'olderalertcmd_send_to_system_status') eq 'true')?$CheckTick:$Check,
    				OBJ_VAL2 => html('Send 4.x alert to SmartView Status') }
    		] },
    		{ SECTION => [
    			{	CP3 => 1,
    				OBJ_VAL1 => (GetKey($table, 'olderalertcmd') ne '')?$CheckTick:$Check,
    				OBJ_VAL2 => html('Run 4.x alert script:'),
    				OBJ_VAL3 => html(stripquotes(GetKey($table, 'olderalertcmd'))) }
    		] }
    	);
    }

	$class{GP_PROPS}=\@props;

	push @classes, \%class;

	# ----------------------------------
	# CLASS: Reporting Tools
	# ----------------------------------
	if ($version gt '6') {
    	my %class;
    	$class{GP_MAINCLASS}='Reporting Tools';
    	$class{GP_SUBCLASS}='';
    	$class{GP_CLASS}='Report';
    	
    	my @props;
    	push @props, (
    		{ SECTION => [
    			{	OBJ_VAL1 => (GetKey($table, 'clp_enable_feature') eq 'true')?$CheckTick:$Check,
    				OBJ_VAL2 => html("Enable tracking all rules (including rules marked as 'None' in the Track column)") }
    		] }
    	);
    
    	if (GetKey($table, 'clp_enable_feature') eq 'true') {
    		my $val;
       		foreach $v1 (GetMembers(GetKey($table, 'clp_global_config:clp_target_server'))) {
    			%var = %{obj2html('_GLOBAL', GetKey($v1, 'Table'), GetKey($v1, 'Name'), $_HREF)};
    			$val .= $var{OBJ_ICON}.' '.$var{OBJ_TEXT}.'<BR/>';
    		}
    		push @props, (
    			{ SECTION => [
    				{	OBJ_VAL1 => html('Send all logs to:'),
    					OBJ_VAL2 => $val }
    			] }
    		);
    
    		my $val;
       		foreach $v1 (GetMembers(GetKey($table, 'clp_global_config:clp_excluded_services'))) {
    			%var = %{obj2html('_GLOBAL', GetKey($v1, 'Table'), GetKey($v1, 'Name'), $_HREF)};
    			$val .= $var{OBJ_ICON}.' '.$var{OBJ_TEXT}.'<BR/>';
       		};
    		push @props, (
    			{ SECTION_BEGIN => html('Services to track') },
    			{ TXT => [
    				{	CLASS => 'note',
    					TEXT => html('All services are included, but for the') }
    			] },
       			{ SF_BEG => [
       				{	SECTION  => html('Excluded Services'),
       					BUTTON   => 1,
       					OBJ_NAME => $class{GP_CLASS},
       					STATE	 => 0 }
      			] },
     			{ SECTION => [ 
     				{ OBJ_VAL1 => $val }
     			] },
       			{ SF_END => 1 },
       			{ SECTION_END => 1 }
      		);
    		
    	}
    
    	$class{GP_PROPS}=\@props;
    
    	push @classes, \%class;
	}

	# ----------------------------------
	# CLASS: OPSEC
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='OPSEC';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='OPSEC';
	
	my @props;
	push @props, (
		{ SECTION_BEGIN => html("OPSEC Product Installation") },
		{ SECTION => [
			{	OBJ_VAL1 => (GetKey($table, 'allow_remote_ra') eq 'true')?$CheckTick:$Check,
				OBJ_VAL2 => html('Allow remote registration of OPSEC products') }
		] },
		{ SECTION_END => 1 }
	);

	$class{GP_PROPS}=\@props;

	push @classes, \%class;

	# ----------------------------------
	# CLASS: SmartCenter Access
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='SmartCenter Access';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='SmartAccess';
	
	my @props;
	push @props, (
		{ SECTION_BEGIN => html("Administrators login restrictions") },
		{ TXT => [
			{	CLASS => 'note_xspc',
				TEXT => html('The following settings apply only to Administrators defined through SmartDashboard.') }
		] },
	);
	
	if (GetKey($table, 'admin_lock_after_bad_attempts_enable') eq 'true') {
		push @props, (
			{ SECTION => [
				{	CP3 => 1,
					OBJ_VAL1 => $CheckTick,
					OBJ_VAL2 => html("Lockout Administrator's account after"),
					OBJ_VAL3 => GetKey($table, 'admin_lock_after_bad_attempts'),
					OBJ_VAL4 => html('login failures') },
				{	INACTIVE => (GetKey($table, 'admin_lock_auto_release_timeout_enable') eq 'true'),
					OBJ_VAL1 => $Spacer,
					OBJ_VAL2 => $Check.
								html(" Unlock Administrator's account") },
				{	INACTIVE => (GetKey($table, 'admin_lock_auto_release_timeout_enable') ne 'true'),
					CP3 => 1,
					OBJ_VAL1 => $Spacer,
					OBJ_VAL2 => $CheckTick.
								html(" Unlock Administrator's account after"),
					OBJ_VAL3 => GetKey($table, 'admin_lock_auto_release_timeout'),
					OBJ_VAL4 => html('minutes') }
			] },
			{ SECTION => [
				{	OBJ_VAL1 => (GetKey($table, 'admin_lock_send_friendly_error_msg') eq 'true')?$CheckTick:$Check,
					OBJ_VAL2 => html("Display a detailed message to a locked out Administrator upon denying access") }
			] }
		);
	} else {
		push @props, (
			{ SECTION => [
				{	OBJ_VAL1 => $Check,
					OBJ_VAL2 => html("Lockout Administrator's account") }
			] }
		);
	}
		
	push @props, { SECTION_END => 1 };

	$class{GP_PROPS}=\@props;

	push @classes, \%class;

	# ----------------------------------
	# CLASS: Non Unique IP Address Ranges
	# ----------------------------------
	my %class;
	$class{GP_MAINCLASS}='Non Unique IP Address Ranges';
	$class{GP_SUBCLASS}='';
	$class{GP_CLASS}='NonUniqueIP';
	
	my @props;
		push @props, (
			{ TXT =>[{	CLASS => 'note_xspc',
						TEXT => html('All addresses that are not in the list will be considered as unique addresses.') }] }
		);
		
		my @section;
		push @section, {	HEAD => 1,
			  				OBJ_VAL1 => html('First IP'),
		  					OBJ_VAL2 => html('Last IP') };
		  					
		@var = GetMembers(GetKey($table, 'private_ip_ranges'));
		foreach $var (@var) {
			push @section, {	OBJ_VAL1 => html(GetKey($var, 'ipaddr_first')),
								OBJ_VAL2 => html(GetKey($var, 'ipaddr_last')) };
		}

		push @props, { TABLE => \@section, C2 => 1 };

	$class{GP_PROPS}=\@props;

	push @classes, \%class;


	# End of Global Properties. Send to templating ...	
	$template->param(GLOBALPROPERTIES => \@classes );

	return $template;
}

1;
